"""Model factories used for tests of contributions (media files)."""

import factory

from geokey.users.tests.model_factories import UserFactory
from geokey.contributions.tests.model_factories import ObservationFactory
from geokey.contributions.models import ImageFile, VideoFile, AudioFile
from geokey.core.tests.helpers.image_helpers import get_image


class ImageFileFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ImageFile

    name = factory.Sequence(lambda n: 'name_%d' % n)
    description = factory.LazyAttribute(lambda o: '%s description' % o.name)
    creator = factory.SubFactory(UserFactory)
    contribution = factory.SubFactory(ObservationFactory)
    image = get_image()


class VideoFileFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = VideoFile

    name = factory.Sequence(lambda n: 'name_%d' % n)
    description = factory.LazyAttribute(lambda o: '%s description' % o.name)
    creator = factory.SubFactory(UserFactory)
    contribution = factory.SubFactory(ObservationFactory)
    youtube_id = 'qb2Cd1--YvY'
    youtube_link = 'http://example.com/98aishfijh'
    swf_link = 'http://example.com/98aishfijh'


class AudioFileFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = AudioFile

    name = factory.Sequence(lambda n: 'name_%d' % n)
    description = factory.LazyAttribute(lambda o: '%s description' % o.name)
    creator = factory.SubFactory(UserFactory)
    contribution = factory.SubFactory(ObservationFactory)
    audio = get_image()



"""Views for superuser tools."""

from django.db.models import Q, Case, When, Sum, IntegerField
from django.views.generic import TemplateView
from django.contrib import messages
from django.contrib.sites.shortcuts import get_current_site

from braces.views import LoginRequiredMixin
from allauth.account.models import EmailAddress
from allauth.socialaccount import providers
from allauth.socialaccount.models import SocialApp
from rest_framework import status
from rest_framework.views import APIView
from rest_framework.response import Response

from geokey.core.decorators import handle_exceptions_for_ajax
from geokey.users.models import User
from geokey.users.serializers import UserSerializer
from geokey.projects.models import Project
from geokey.contributions.base import OBSERVATION_STATUS
from geokey.superusertools.base import IsSuperuser
from geokey.superusertools.mixins import SuperuserMixin


# #############################################################################
#
# ADMIN VIEWS
#
# #############################################################################

class ManageSuperusers(LoginRequiredMixin, SuperuserMixin, TemplateView):
    """Manage superusers page."""

    template_name = 'superusertools/manage_superusers.html'

    def get_context_data(self):
        """
        Return the context to render the view.

        Adds a list of superusers to the context.

        Returns
        -------
        dict
        """
        return {
            'superusers': User.objects.filter(
                is_superuser=True).only('display_name')
        }


class ManageInactiveUsers(LoginRequiredMixin, SuperuserMixin, TemplateView):
    """Manage inactive users page."""

    template_name = 'superusertools/manage_inactive_users.html'

    def get_context_data(self):
        """
        Return the context to render the view.

        Add a list of inactive users to the context.

        Returns
        -------
        dict
        """
        return {'inactive_users': User.objects.filter(
            is_active=False).defer('is_superuser')}

    def post(self, request):
        """
        Activate inactive users.

        Parameters
        ----------
        request : django.http.HttpRequest
            Object representing the request.

        Returns
        -------
        django.http.HttpResponse
        """
        data = request.POST
        context = self.get_context_data()
        inactive_users = context.get('inactive_users')

        if inactive_users:
            active_users = inactive_users.filter(
                id__in=data.getlist('activate_users'))
            in_total = len(active_users)

            for email in EmailAddress.objects.filter(user__in=active_users):
                email.verified = True
                email.set_as_primary(conditional=True)
                email.save()

            active_users.update(is_active=True)
            messages.success(
                self.request,
                '%s user(s) has been activated.' % in_total)
            context['inactive_users'] = User.objects.filter(is_active=False)

        return self.render_to_response(context)


class ManageProjects(LoginRequiredMixin, SuperuserMixin, TemplateView):
    """Manage projects page."""

    template_name = 'superusertools/manage_projects.html'

    def get_context_data(self):
        """
        Return the context to render the view.

        Add a list of projects to the context (with numbers in total of
        contributions, comments, media files).

        Returns
        -------
        dict
        """
        return {'projects': Project.objects.all().annotate(
            contributions_count=Sum(Case(When(
                ~Q(observations__status=OBSERVATION_STATUS.deleted) &
                Q(observations__isnull=False),
                then=1
            ), default=0, output_field=IntegerField()), distinct=True),
            comments_count=Sum(Case(When(
                ~Q(observations__status=OBSERVATION_STATUS.deleted) &
                Q(observations__isnull=False),
                then='observations__num_comments'
            ), default=0, output_field=IntegerField()), distinct=True),
            media_count=Sum(Case(When(
                ~Q(observations__status=OBSERVATION_STATUS.deleted) &
                Q(observations__isnull=False),
                then='observations__num_media'
            ), default=0, output_field=IntegerField()), distinct=True)
        ).defer(
            'description',
            'everyone_contributes',
            'admins',
            'geographic_extent'
        )}


class PlatformSettings(LoginRequiredMixin, SuperuserMixin, TemplateView):
    """Platform settings page."""

    template_name = 'superusertools/platform_settings.html'

    def get_context_data(self):
        """
        Return the context to render the view.

        Add a current site to the context.

        Returns
        -------
        dict
        """
        return {'site': get_current_site(self.request)}

    def post(self, request):
        """
        Handle POST request.

        Update the platform settings.

        Parameters
        ----------
        request : django.http.HttpRequest
            Object representing the request.

        Returns
        -------
        django.http.HttpResponse
        """
        data = request.POST
        context = self.get_context_data()
        site = context.get('site')

        if site:
            site.name = data.get('name')
            site.domain = data.get('domain')
            site.save()
            messages.success(
                self.request,
                'Platform settings have been updated.'
            )
            context['site'] = site

        return self.render_to_response(context)


class ProviderList(LoginRequiredMixin, SuperuserMixin, TemplateView):
    """A list of all providers page."""

    template_name = 'superusertools/provider_list.html'

    def get_context_data(self):
        """
        Return the context to render the view.

        Add all providers to the context.

        Returns
        -------
        dict
        """
        return {'providers': providers.registry.get_list()}


class ProviderContext(LoginRequiredMixin, SuperuserMixin):
    """Context mixin that adds a provider to render the template."""

    def get_context_data(self, provider_id, *args, **kwargs):
        """
        Return the context to render the view.

        Add a provider and optional social app to the context.

        Parameters
        ----------
        provider_id : str
            Identifies the provider in the list of available providers.

        Returns
        -------
        dict
        """
        try:
            provider = providers.registry.by_id(provider_id)

            try:
                social_app = SocialApp.objects.get_current(
                    provider.id,
                    self.request
                )
            except SocialApp.DoesNotExist:
                social_app = None

            return super(ProviderContext, self).get_context_data(
                provider=provider,
                social_app=social_app,
                *args,
                **kwargs
            )
        except:
            return {
                'error': 'Not found.',
                'error_description': 'Provider not found.'
            }


class ProviderOverview(ProviderContext, TemplateView):
    """Overview of a provider page."""

    template_name = 'superusertools/provider_overview.html'

    def post(self, request, provider_id):
        """
        Handle POST request.

        Update or enable the social app.

        Parameters
        ----------
        request : django.http.HttpRequest
            Object representing the request.
        provider_id : str
            Identifies the provider in the list of available providers.

        Returns
        -------
        django.http.HttpResponse
        """
        data = request.POST
        context = self.get_context_data(provider_id)
        provider = context.get('provider')
        social_app = context.get('social_app')

        if social_app:
            social_app.client_id = data.get('client_id')
            social_app.secret = data.get('secret')
            social_app.key = data.get('key')
            social_app.save()
            messages.success(
                self.request,
                'Provider has been updated.'
            )
        else:
            social_app = SocialApp.objects.create(
                provider=provider.id,
                name=provider.name,
                client_id=data.get('client_id'),
                secret=data.get('secret'),
                key=data.get('key')
            )
            social_app.sites.add(get_current_site(request))
            messages.success(
                self.request,
                'Provider has been activated.'
            )

        context['social_app'] = social_app

        return self.render_to_response(context)


# #############################################################################
#
# AJAX API
#
# #############################################################################

class SuperusersAjaxView(APIView):
    """Ajax API for all superusers."""

    permission_classes = (IsSuperuser,)

    @handle_exceptions_for_ajax
    def post(self, request):
        """
        Handle POST request.

        Add a user to superusers.

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request.

        Returns
        -------
        rest_framework.response.Response
            Response to the request.
        """
        user = User.objects.get(pk=request.data.get('user_id'))
        user.is_superuser = True
        user.save()

        serializer = UserSerializer(
            User.objects.filter(is_superuser=True),
            many=True
        )
        return Response(
            {'users': serializer.data},
            status=status.HTTP_201_CREATED
        )


class SingleSuperuserAjaxView(APIView):
    """Ajax API for a single superuser."""

    permission_classes = (IsSuperuser,)

    @handle_exceptions_for_ajax
    def delete(self, request, user_id):
        """
        Handle DELETE request.

        Remove a user from superusers.

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request.
        user_id : int
            Identifies the user in the database.

        Returns
        -------
        rest_framework.response.Response
            Response to the request.
        """
        user = User.objects.get(pk=user_id, is_superuser=True)
        user.is_superuser = False
        user.save()

        return Response(status=status.HTTP_204_NO_CONTENT)



"""Views for media files of contributions."""

from django.core.exceptions import PermissionDenied

from rest_framework import status
from rest_framework.views import APIView
from rest_framework.response import Response

from geokey.core.decorators import handle_exceptions_for_ajax
from geokey.core.exceptions import MalformedRequestData
from geokey.users.models import User

from .base import SingleAllContribution
from ..models import MediaFile
from ..serializers import FileSerializer


class MediaAbstractAPIView(APIView):
    """Abstract class for media."""

    def get_user(self, request):
        """
        Get user of a request.

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request.

        Returns
        -------
        geokey.users.models.User
            User of a request.
        """
        user = request.user

        if user.is_anonymous():
            user = User.objects.get(display_name='AnonymousUser')

        return user

    def get_file(self, contribution, file_id):
        """
        Get media file of a contribution.

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request.
        contribution : geokey.contributions.models.Observation
            Contribution to retrieve the file from.
        file_id : int
            Identifies the media file in the database.

        Returns
        -------
        geokey.contributions.models.MediaFile
            Media file of a contribution.
        """
        return contribution.files_attached\
            .select_related('creator')\
            .get(pk=file_id)

    def get_list_and_respond(self, request, contribution):
        """
        Respond to a GET request with a list of all media files.

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request.
        contribution : geokey.contributions.models.Observation
            Contribution the media files are requested for.

        Returns
        -------
        rest_framework.response.Respones
            Contains the serialized media files.
        """
        serializer = FileSerializer(
            contribution.files_attached.all(),
            many=True,
            context={'user': self.get_user(request)}
        )
        return Response(serializer.data, status=status.HTTP_200_OK)

    def create_and_respond(self, request, contribution):
        """
        Respond to a POST request by creating a media file.

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request.
        contribution : geokey.contributions.models.Observation
            Contribution the media file should be added to.

        Returns
        -------
        rest_framework.response.Respone
            Contains the serialized media file.

        Raises
        ------
        MalformedRequestData
            When name is not set or file is not attached.
        PermissionDenied
            When user is not allowed to contribute to the project.
        """
        user = self.get_user(request)

        data = self.request.POST
        name = data.get('name')
        description = data.get('description')
        file = self.request.FILES.get('file')

        errors = []
        if name is None:
            errors.append('Property `name` is not set')
        if file is None:
            errors.append('No file attached')
        if errors:
            raise MalformedRequestData('%s.' % ', '.join(errors))

        if contribution.project.can_contribute(user):
            file = MediaFile.objects.create(
                name=name,
                description=description,
                contribution=contribution,
                creator=user,
                the_file=file
            )

            serializer = FileSerializer(file, context={'user': user})
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        else:
            raise PermissionDenied(
                'You are not allowed to contribute to the project.'
            )

    def get_single_and_respond(self, request, file):
        """
        Respond to a GET request with a single media file.

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request.
        file : geokey.contributions.models.MediaFile
            Media file to be returned.

        Returns
        -------
        rest_framework.response.Respone
            Contains the serialized media file.
        """
        serializer = FileSerializer(
            file,
            context={'user': self.get_user(request)}
        )
        return Response(serializer.data, status=status.HTTP_200_OK)

    def delete_and_respond(self, request, contribution, file):
        """
        Respond to a DELETE request by deleting the media file.

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request.
        contribution : geokey.contributions.models.Observation
            Contribution of a media file.
        file : geokey.contributions.models.MediaFile
            Media file to be deleted.

        Returns
        -------
        rest_framework.response.Respone
            Empty response indicating success.

        Raises
        ------
        PermissionDenied
            When user is not allowed to delete the media file.
        """
        user = self.get_user(request)

        if file.creator == user or contribution.project.can_moderate(user):
            file.delete()
            return Response(status=status.HTTP_204_NO_CONTENT)
        else:
            raise PermissionDenied(
                'You are neither the creator if this file nor a project '
                'moderator and therefore not eligable to delete it.'
            )


class MediaAPIView(SingleAllContribution, MediaAbstractAPIView):
    """Public API for all media files."""

    @handle_exceptions_for_ajax
    def get(self, request, project_id, contribution_id):
        """
        Handle GET request.

        Return a list of all media files of the contribution.

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request.
        project_id : int
            Identifies the project in the database.
        contribution_id : int
            Identifies the contribution in the database.

        Returns
        -------
        rest_framework.response.Respone
            Contains the serialised media files.
        """
        contribution = self.get_contribution(
            request.user,
            project_id,
            contribution_id
        )

        return self.get_list_and_respond(request, contribution)

    @handle_exceptions_for_ajax
    def post(self, request, project_id, contribution_id):
        """
        Handle POST request.

        Add a new media file to the contribution.

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request.
        project_id : int
            Identifies the project in the database.
        contribution_id : int
            Identifies the contribution in the database.

        Returns
        -------
        rest_framework.response.Respone
            Contains the serialised media file.
        """
        contribution = self.get_contribution(
            request.user,
            project_id,
            contribution_id
        )

        return self.create_and_respond(request, contribution)


class SingleMediaAPIView(SingleAllContribution, MediaAbstractAPIView):
    """Public API for a single media."""

    @handle_exceptions_for_ajax
    def get(self, request, project_id, contribution_id, file_id):
        """
        Handle GET request.

        Return a single media file.

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request.
        project_id : int
            Identifies the project in the database.
        contribution_id : int
            Identifies the contribution in the database.
        file_id : int
            Identifies the media file in the database.

        Returns
        -------
        rest_framework.response.Respone
            Contains the serialized media file.
        """
        contribution = self.get_contribution(
            request.user,
            project_id,
            contribution_id
        )
        file = self.get_file(contribution, file_id)

        return self.get_single_and_respond(request, file)

    @handle_exceptions_for_ajax
    def delete(self, request, project_id, contribution_id, file_id):
        """
        Handle DELETE request.

        Delete the media file.

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request.
        project_id : int
            Identifies the project in the database.
        contribution_id : int
            Identifies the contribution in the database.
        file_id : int
            Identifies the media file in the database.

        Returns
        -------
        rest_framework.response.Respone
            Empty response indicating success.
        """
        contribution = self.get_contribution(
            request.user,
            project_id,
            contribution_id
        )
        file = self.get_file(contribution, file_id)

        return self.delete_and_respond(request, contribution, file)



"""Managers for users."""

from django.db import models
from django.conf import settings
from django.contrib.auth.models import AbstractBaseUser
from django.utils import timezone
from django.dispatch import receiver

from simple_history.models import HistoricalRecords

from django_pgjson.fields import JsonBField
from oauth2_provider.models import AccessToken
from allauth.account.signals import email_confirmed

from geokey.core.mixins import FilterMixin
from .managers import UserManager


class User(AbstractBaseUser):
    """
    A user registered in the platform.
    """
    email = models.EmailField(
        unique=True,
        error_messages={
            'unique': 'A user is already registered with this email address.'
        }
    )
    display_name = models.CharField(
        max_length=50,
        unique=True,
        error_messages={
            'unique': 'A user is already registered with this display name.'
        }
    )
    date_joined = models.DateTimeField(default=timezone.now)
    is_active = models.BooleanField(default=True)
    is_superuser = models.BooleanField(default=False)

    USERNAME_FIELD = 'display_name'
    REQUIRED_FIELDS = ['email']

    objects = UserManager()

    def reset_password(self, password):
        """
        Resets the users password and deletes all access tokens assigned to the
        user.

        Parameters
        ----------
        password : str
            New password of the user
        """
        self.set_password(password)
        self.save()
        AccessToken.objects.filter(user=self).delete()


@receiver(email_confirmed)
def set_user_active(sender, **kwargs):
    email_address = kwargs.get('email_address')
    email_address.user.is_active = True
    email_address.user.save()


class UserGroup(FilterMixin, models.Model):
    """
    A user gropup assigned to a project.
    """
    name = models.CharField(max_length=100)
    description = models.TextField(blank=True, null=True)
    users = models.ManyToManyField(settings.AUTH_USER_MODEL)
    project = models.ForeignKey('projects.Project', related_name='usergroups')
    can_contribute = models.BooleanField(default=True)
    can_moderate = models.BooleanField(default=False)
    filters = JsonBField(blank=True, null=True)
    where_clause = models.TextField(blank=True, null=True)
    history = HistoricalRecords()

    def save(self, *args, **kwargs):
        """
        Overwrites save to implement integrity ensurance.
        """
        if self.can_moderate:
            self.can_contribute = True

        super(UserGroup, self).save(*args, **kwargs)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('contributions', '0015_auto_20150907_1345'),
    ]

    operations = [
        migrations.CreateModel(
            name='AudioFile',
            fields=[
                ('mediafile_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='contributions.MediaFile')),
                ('audio', models.FileField(upload_to=b'user-uploads/audio')),
            ],
            options={
                'ordering': ['id'],
            },
            bases=('contributions.mediafile',),
        ),
    ]



"""Core signals."""

from django.dispatch import Signal


class RequestAccessorSignal(Signal):
    """Request accessor signal."""

    def __init__(self, providing_args=None):
        """Initiate the signal."""
        return Signal.__init__(self, providing_args)

    def connect(self, receiver, sender=None, weak=True, dispatch_uid=None):
        """Connect the signal."""
        Signal.connect(self, receiver, sender, weak, dispatch_uid)


request_accessor = RequestAccessorSignal()


def get_request():
    """Get the current request."""
    entry = request_accessor.send(None)
    if entry:
        entry = entry[0]
        if entry:
            return entry[1]
    return None



"""Tests for commands of users."""

from pytz import utc
from datetime import datetime

from django.test import TestCase

from allauth.account.models import EmailAddress, EmailConfirmation

from ..models import User
from ..management.commands.check_confirm import Command as CheckConfirm
from ..tests.model_factories import UserFactory


class CheckConfirmTest(TestCase):
    def test_check_confirm(self):
        confirmed_user = UserFactory.create()
        unconfirmed_user = UserFactory.create()

        confirmed_email = EmailAddress.objects.create(
            user=confirmed_user,
            email=confirmed_user.email,
            verified=True
        )

        unconfirmed_email = EmailAddress.objects.create(
            user=unconfirmed_user,
            email=unconfirmed_user.email,
            verified=False
        )

        EmailConfirmation.objects.create(
            email_address=confirmed_email,
            created=datetime.utcnow().replace(tzinfo=utc),
            sent=datetime.utcnow().replace(tzinfo=utc),
            key='38974hewjkhnuweyf8h8d'
        )
        EmailConfirmation.objects.create(
            email_address=unconfirmed_email,
            created=datetime.utcnow().replace(tzinfo=utc),
            sent=datetime.utcnow().replace(tzinfo=utc),
            key='38974hewjkhnsdfuweyf8h8d'
        )

        yesterday = datetime.utcnow().replace(tzinfo=utc)
        command = CheckConfirm()
        command.set_user_inactive(yesterday)

        ref_unconfirmed = User.objects.get(pk=unconfirmed_user.id)
        self.assertFalse(ref_unconfirmed.is_active)

        ref_confirmed = User.objects.get(pk=confirmed_user.id)
        self.assertTrue(ref_confirmed.is_active)



"""Template tags for project attributes."""

from django import template
from django.template.loader import render_to_string


register = template.Library()


@register.simple_tag
def project_attributes(project):
    return render_to_string(
        'projects/project_attributes.html',
        {
            'creator': project.creator.display_name,
            'created_at': project.created_at.strftime("%d %B %Y, %H:%M"),
            'private_label': ('Private' if project.isprivate else 'Public'),
            'inactive': project.status == 'inactive'
        }
    )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.db.models.deletion
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('categories', '0013_auto_20150130_1440'),
        ('projects', '0005_auto_20150202_1041'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contributions', '0012_auto_20150807_0854'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='historicalobservation',
            name='category_id',
        ),
        migrations.RemoveField(
            model_name='historicalobservation',
            name='creator_id',
        ),
        migrations.RemoveField(
            model_name='historicalobservation',
            name='location_id',
        ),
        migrations.RemoveField(
            model_name='historicalobservation',
            name='project_id',
        ),
        migrations.RemoveField(
            model_name='historicalobservation',
            name='updator_id',
        ),
        migrations.AddField(
            model_name='historicalobservation',
            name='category',
            field=models.ForeignKey(related_name='+', on_delete=django.db.models.deletion.DO_NOTHING, db_constraint=False, blank=True, to='categories.Category', null=True),
        ),
        migrations.AddField(
            model_name='historicalobservation',
            name='creator',
            field=models.ForeignKey(related_name='+', on_delete=django.db.models.deletion.DO_NOTHING, db_constraint=False, blank=True, to=settings.AUTH_USER_MODEL, null=True),
        ),
        migrations.AddField(
            model_name='historicalobservation',
            name='location',
            field=models.ForeignKey(related_name='+', on_delete=django.db.models.deletion.DO_NOTHING, db_constraint=False, blank=True, to='contributions.Location', null=True),
        ),
        migrations.AddField(
            model_name='historicalobservation',
            name='project',
            field=models.ForeignKey(related_name='+', on_delete=django.db.models.deletion.DO_NOTHING, db_constraint=False, blank=True, to='projects.Project', null=True),
        ),
        migrations.AddField(
            model_name='historicalobservation',
            name='search_index',
            field=models.TextField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='historicalobservation',
            name='updator',
            field=models.ForeignKey(related_name='+', on_delete=django.db.models.deletion.DO_NOTHING, db_constraint=False, blank=True, to=settings.AUTH_USER_MODEL, null=True),
        ),
        migrations.AddField(
            model_name='observation',
            name='search_index',
            field=models.TextField(null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='historicalobservation',
            name='history_user',
            field=models.ForeignKey(related_name='+', on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, null=True),
        ),
        migrations.AlterField(
            model_name='historicalobservation',
            name='search_matches',
            field=models.TextField(null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='observation',
            name='search_matches',
            field=models.TextField(null=True, blank=True),
        ),
    ]



"""Mixins for extensions."""

from geokey.superusertools.mixins import SuperuserMixin


class SuperuserMixin(SuperuserMixin):
    """A mixin for superuser."""

    exception_message = 'This extension is for superusers only.'



"""Template tags for social features."""

from django import template
from django.db.models import Q

from allauth.socialaccount import providers
from allauth.socialaccount.models import SocialApp


register = template.Library()


@register.assignment_tag
def get_social_apps():
    """Get all enabled social apps."""
    social_apps = SocialApp.objects.exclude(Q(client_id__exact='')).distinct()

    for social_app in social_apps:
        try:
            provider = providers.registry.by_id(social_app.provider)
        except:
            provider = None

        social_app.provider = provider

    return social_apps



"""Core exceptions."""


class Unauthenticated(Exception):
    """Thrown when user is not authenticated."""

    pass


class MalformedRequestData(Exception):
    """Thrown when request data is malformed."""

    pass


class InputError(Exception):
    """Thrown on input error."""

    pass


class FileTypeError(Exception):
    """Thrown on file type error."""

    pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0001_squashed_0004_auto_20150617_0902'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='display_name',
            field=models.CharField(unique=True, max_length=50, error_messages={b'unique': b'A user is already registered with this display name.'}),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='user',
            name='email',
            field=models.EmailField(unique=True, max_length=75, error_messages={b'unique': b'A user is already registered with this email address.'}),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0002_auto_20150824_1603'),
        ('oauth2_provider', '0002_08_updates'),
    ]

    operations = [
        migrations.RunSQL('DROP TABLE IF EXISTS oauth2_provider_application;')
    ]



"""Tests for models of contributions (comments)."""

from django.test import TestCase

from nose.tools import raises

from geokey.contributions.models import Comment, post_save_comment_count_update
from ..model_factories import ObservationFactory, CommentFactory


class TestCommentPostSave(TestCase):
    def test_post_save_comment_count_update(self):
        observation = ObservationFactory()
        CommentFactory.create_batch(5, **{'commentto': observation})
        comment = CommentFactory.create(**{
            'commentto': observation,
            'status': 'deleted'
        })

        post_save_comment_count_update(Comment, instance=comment)
        self.assertEqual(comment.commentto.num_media, 0)
        self.assertEqual(comment.commentto.num_comments, 5)


class CommentTest(TestCase):
    @raises(Comment.DoesNotExist)
    def test_delete_comment(self):
        comment = CommentFactory()
        comment.delete()
        Comment.objects.get(pk=comment.id)

    def test_delete_nested(self):
        comment = CommentFactory()
        response = CommentFactory(**{'respondsto': comment})
        CommentFactory.create_batch(3, **{'respondsto': response})
        self.assertEquals(len(Comment.objects.all()), 5)
        comment.delete()
        self.assertEquals(len(Comment.objects.all()), 0)



"""Tests for logger: model UserGroup."""

from django.test import TestCase

from geokey.core.models import LoggerHistory
from geokey.users.tests.model_factories import UserFactory
from geokey.projects.tests.model_factories import ProjectFactory
from geokey.users.tests.model_factories import UserGroupFactory


class LogUserGroupTest(TestCase):
    """Test model UserGroup."""

    def setUp(self):
        """Set up test."""
        self.user = UserFactory.create()
        self.project = ProjectFactory.create(**{
            'creator': self.user})
        self.usergroup = UserGroupFactory.create(**{
            'project': self.project})

    def test_log_create(self):
        """Test when user group gets created."""
        log_count_init = LoggerHistory.objects.count()
        usergroup = UserGroupFactory.create(**{
            'project': self.project})

        log = LoggerHistory.objects.last()
        log_count = LoggerHistory.objects.count()

        self.assertNotEqual(log.user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(log.project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(log.usergroup, {
            'id': str(usergroup.id),
            'name': usergroup.name})
        self.assertEqual(log.category, None)
        self.assertEqual(log.field, None)
        self.assertEqual(log.location, None)
        self.assertEqual(log.observation, None)
        self.assertEqual(log.comment, None)
        self.assertEqual(log.subset, None)
        self.assertEqual(log.action, {
            'id': 'created',
            'class': 'UserGroup'})
        self.assertEqual(log_count, log_count_init + 1)
        self.assertEqual(log.historical, None)

    def test_log_delete(self):
        """Test when user group gets deleted."""
        usergroup_id = self.usergroup.id
        usergroup_name = self.usergroup.name
        log_count_init = LoggerHistory.objects.count()
        self.usergroup.delete()

        log = LoggerHistory.objects.last()
        log_count = LoggerHistory.objects.count()

        self.assertNotEqual(log.user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(log.project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(log.usergroup, {
            'id': str(usergroup_id),
            'name': usergroup_name})
        self.assertEqual(log.category, None)
        self.assertEqual(log.field, None)
        self.assertEqual(log.location, None)
        self.assertEqual(log.observation, None)
        self.assertEqual(log.comment, None)
        self.assertEqual(log.subset, None)
        self.assertEqual(log.action, {
            'id': 'deleted',
            'class': 'UserGroup'})
        self.assertEqual(log_count, log_count_init + 1)
        self.assertEqual(log.historical, None)

    def test_log_update_name(self):
        """Test when name changes."""
        log_count_init = LoggerHistory.objects.count()
        original_name = self.usergroup.name
        self.usergroup.name = '%s UPDATED' % self.usergroup.name
        self.usergroup.save()

        log = LoggerHistory.objects.last()
        log_count = LoggerHistory.objects.count()

        self.assertNotEqual(log.user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(log.project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(log.usergroup, {
            'id': str(self.usergroup.id),
            'name': self.usergroup.name})
        self.assertEqual(log.category, None)
        self.assertEqual(log.field, None)
        self.assertEqual(log.location, None)
        self.assertEqual(log.observation, None)
        self.assertEqual(log.comment, None)
        self.assertEqual(log.subset, None)
        self.assertEqual(log.action, {
            'id': 'updated',
            'class': 'UserGroup',
            'field': 'name'})
        self.assertEqual(log_count, log_count_init + 1)
        history = self.usergroup.history.get(pk=log.historical.get('id'))
        self.assertEqual(history.id, self.usergroup.id)
        self.assertEqual(history.name, original_name)

    def test_log_update_can_contribute(self):
        """Test when setting gets set to `can contribute`."""
        self.usergroup.can_contribute = False
        self.usergroup.can_moderate = False
        self.usergroup.save()
        log_count_init = LoggerHistory.objects.count()
        original_can_contribute = self.usergroup.can_contribute
        original_can_moderate = self.usergroup.can_moderate
        self.usergroup.can_contribute = True
        self.usergroup.can_moderate = False
        self.usergroup.save()

        log = LoggerHistory.objects.last()
        log_count = LoggerHistory.objects.count()

        self.assertNotEqual(log.user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(log.project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(log.usergroup, {
            'id': str(self.usergroup.id),
            'name': self.usergroup.name})
        self.assertEqual(log.category, None)
        self.assertEqual(log.field, None)
        self.assertEqual(log.location, None)
        self.assertEqual(log.observation, None)
        self.assertEqual(log.comment, None)
        self.assertEqual(log.subset, None)
        self.assertEqual(log.action, {
            'id': 'updated',
            'class': 'UserGroup',
            'field': 'can_contribute',
            'value': 'True'})
        self.assertEqual(log_count, log_count_init + 1)
        history = self.usergroup.history.get(pk=log.historical.get('id'))
        self.assertEqual(history.id, self.usergroup.id)
        self.assertEqual(history.can_contribute, original_can_contribute)
        self.assertEqual(history.can_moderate, original_can_moderate)

        self.usergroup.can_contribute = False
        self.usergroup.can_moderate = True
        self.usergroup.save()
        log_count_init = LoggerHistory.objects.count()
        original_can_contribute = self.usergroup.can_contribute
        original_can_moderate = self.usergroup.can_moderate
        self.usergroup.can_contribute = True
        self.usergroup.can_moderate = False
        self.usergroup.save()

        log = LoggerHistory.objects.last()
        log_count = LoggerHistory.objects.count()

        self.assertNotEqual(log.user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(log.project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(log.usergroup, {
            'id': str(self.usergroup.id),
            'name': self.usergroup.name})
        self.assertEqual(log.category, None)
        self.assertEqual(log.field, None)
        self.assertEqual(log.location, None)
        self.assertEqual(log.observation, None)
        self.assertEqual(log.comment, None)
        self.assertEqual(log.subset, None)
        self.assertEqual(log.action, {
            'id': 'updated',
            'class': 'UserGroup',
            'field': 'can_contribute',
            'value': 'True'})
        self.assertEqual(log_count, log_count_init + 1)
        history = self.usergroup.history.get(pk=log.historical.get('id'))
        self.assertEqual(history.id, self.usergroup.id)
        self.assertEqual(history.can_contribute, original_can_contribute)
        self.assertEqual(history.can_moderate, original_can_moderate)

    def test_log_update_can_moderate(self):
        """Test when setting gets set to `can moderate`."""
        self.usergroup.can_contribute = False
        self.usergroup.can_moderate = False
        self.usergroup.save()
        log_count_init = LoggerHistory.objects.count()
        original_can_contribute = self.usergroup.can_contribute
        original_can_moderate = self.usergroup.can_moderate
        self.usergroup.can_contribute = True
        self.usergroup.can_moderate = True
        self.usergroup.save()

        log = LoggerHistory.objects.last()
        log_count = LoggerHistory.objects.count()

        self.assertNotEqual(log.user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(log.project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(log.usergroup, {
            'id': str(self.usergroup.id),
            'name': self.usergroup.name})
        self.assertEqual(log.category, None)
        self.assertEqual(log.field, None)
        self.assertEqual(log.location, None)
        self.assertEqual(log.observation, None)
        self.assertEqual(log.comment, None)
        self.assertEqual(log.subset, None)
        self.assertEqual(log.action, {
            'id': 'updated',
            'class': 'UserGroup',
            'field': 'can_moderate',
            'value': 'True'})
        self.assertEqual(log_count, log_count_init + 1)
        history = self.usergroup.history.get(pk=log.historical.get('id'))
        self.assertEqual(history.id, self.usergroup.id)
        self.assertEqual(history.can_contribute, original_can_contribute)
        self.assertEqual(history.can_moderate, original_can_moderate)

        self.usergroup.can_contribute = True
        self.usergroup.can_moderate = False
        self.usergroup.save()
        log_count_init = LoggerHistory.objects.count()
        original_can_contribute = self.usergroup.can_contribute
        original_can_moderate = self.usergroup.can_moderate
        self.usergroup.can_contribute = True
        self.usergroup.can_moderate = True
        self.usergroup.save()

        log = LoggerHistory.objects.last()
        log_count = LoggerHistory.objects.count()

        self.assertNotEqual(log.user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(log.project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(log.usergroup, {
            'id': str(self.usergroup.id),
            'name': self.usergroup.name})
        self.assertEqual(log.category, None)
        self.assertEqual(log.field, None)
        self.assertEqual(log.location, None)
        self.assertEqual(log.observation, None)
        self.assertEqual(log.comment, None)
        self.assertEqual(log.subset, None)
        self.assertEqual(log.action, {
            'id': 'updated',
            'class': 'UserGroup',
            'field': 'can_moderate',
            'value': 'True'})
        self.assertEqual(log_count, log_count_init + 1)
        history = self.usergroup.history.get(pk=log.historical.get('id'))
        self.assertEqual(history.id, self.usergroup.id)
        self.assertEqual(history.can_contribute, original_can_contribute)
        self.assertEqual(history.can_moderate, original_can_moderate)

    def test_log_update_can_view(self):
        """Test when setting gets set to `can view`."""
        self.usergroup.can_contribute = True
        self.usergroup.can_moderate = False
        self.usergroup.save()
        log_count_init = LoggerHistory.objects.count()
        original_can_contribute = self.usergroup.can_contribute
        original_can_moderate = self.usergroup.can_moderate
        self.usergroup.can_contribute = False
        self.usergroup.can_moderate = False
        self.usergroup.save()

        log = LoggerHistory.objects.last()
        log_count = LoggerHistory.objects.count()

        self.assertNotEqual(log.user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(log.project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(log.usergroup, {
            'id': str(self.usergroup.id),
            'name': self.usergroup.name})
        self.assertEqual(log.category, None)
        self.assertEqual(log.field, None)
        self.assertEqual(log.location, None)
        self.assertEqual(log.observation, None)
        self.assertEqual(log.comment, None)
        self.assertEqual(log.subset, None)
        self.assertEqual(log.action, {
            'id': 'updated',
            'class': 'UserGroup',
            'field': 'can_view',
            'value': 'True'})
        self.assertEqual(log_count, log_count_init + 1)
        history = self.usergroup.history.get(pk=log.historical.get('id'))
        self.assertEqual(history.id, self.usergroup.id)
        self.assertEqual(history.can_contribute, original_can_contribute)
        self.assertEqual(history.can_moderate, original_can_moderate)

        self.usergroup.can_contribute = True
        self.usergroup.can_moderate = True
        self.usergroup.save()
        log_count_init = LoggerHistory.objects.count()
        original_can_contribute = self.usergroup.can_contribute
        original_can_moderate = self.usergroup.can_moderate
        self.usergroup.can_contribute = False
        self.usergroup.can_moderate = False
        self.usergroup.save()

        log = LoggerHistory.objects.last()
        log_count = LoggerHistory.objects.count()

        self.assertNotEqual(log.user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(log.project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(log.usergroup, {
            'id': str(self.usergroup.id),
            'name': self.usergroup.name})
        self.assertEqual(log.category, None)
        self.assertEqual(log.field, None)
        self.assertEqual(log.location, None)
        self.assertEqual(log.observation, None)
        self.assertEqual(log.comment, None)
        self.assertEqual(log.subset, None)
        self.assertEqual(log.action, {
            'id': 'updated',
            'class': 'UserGroup',
            'field': 'can_view',
            'value': 'True'})
        self.assertEqual(log_count, log_count_init + 1)
        history = self.usergroup.history.get(pk=log.historical.get('id'))
        self.assertEqual(history.id, self.usergroup.id)
        self.assertEqual(history.can_contribute, original_can_contribute)
        self.assertEqual(history.can_moderate, original_can_moderate)

    def test_log_add_user(self):
        """Test when user is added."""
        log_count_init = LoggerHistory.objects.count()
        new_user = UserFactory.create()
        self.usergroup.users.add(new_user)

        log = LoggerHistory.objects.last()
        log_count = LoggerHistory.objects.count()

        self.assertNotEqual(log.user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(log.project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(log.usergroup, {
            'id': str(self.usergroup.id),
            'name': self.usergroup.name})
        self.assertEqual(log.category, None)
        self.assertEqual(log.field, None)
        self.assertEqual(log.location, None)
        self.assertEqual(log.observation, None)
        self.assertEqual(log.comment, None)
        self.assertEqual(log.subset, None)
        self.assertEqual(log.action, {
            'id': 'updated',
            'class': 'UserGroup_users',
            'subaction': 'add',
            'user_id': str(new_user.id),
            'user_display_name': new_user.display_name})
        self.assertEqual(log_count, log_count_init + 1)
        self.assertEqual(log.historical, None)

    def test_log_remove_user(self):
        """Test when user is removed."""
        existing_user = UserFactory.create()
        self.usergroup.users.add(existing_user)
        log_count_init = LoggerHistory.objects.count()
        self.usergroup.users.remove(existing_user)

        log = LoggerHistory.objects.last()
        log_count = LoggerHistory.objects.count()

        self.assertNotEqual(log.user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(log.project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(log.usergroup, {
            'id': str(self.usergroup.id),
            'name': self.usergroup.name})
        self.assertEqual(log.category, None)
        self.assertEqual(log.field, None)
        self.assertEqual(log.location, None)
        self.assertEqual(log.observation, None)
        self.assertEqual(log.comment, None)
        self.assertEqual(log.subset, None)
        self.assertEqual(log.action, {
            'id': 'updated',
            'class': 'UserGroup_users',
            'subaction': 'remove',
            'user_id': str(existing_user.id),
            'user_display_name': existing_user.display_name})
        self.assertEqual(log_count, log_count_init + 1)
        self.assertEqual(log.historical, None)



"""Serializers for subsets."""

from rest_framework.serializers import ModelSerializer

from geokey.subsets.models import Subset


class SubsetSerializer(ModelSerializer):
    """Serializer for a subset."""

    class Meta:
        """Serializer meta."""

        model = Subset
        depth = 1
        fields = ('id', 'name', 'description', 'creator')



"""Views for categories."""

from django.views.generic import CreateView, TemplateView
from django.shortcuts import redirect
from django.core.urlresolvers import reverse
from django.contrib import messages
from django.template.defaultfilters import slugify
from django.utils.html import strip_tags
from django.utils.safestring import mark_safe

from rest_framework import status
from rest_framework.views import APIView
from rest_framework.response import Response

from braces.views import LoginRequiredMixin

from geokey.projects.models import Project
from geokey.projects.views import ProjectContext
from geokey.core.decorators import handle_exceptions_for_ajax

from .base import STATUS
from .models import (
    Category, Field, TextField, NumericField, LookupField, LookupValue,
    MultipleLookupField, MultipleLookupValue
)
from .forms import CategoryCreateForm, FieldCreateForm
from .serializers import (
    CategorySerializer, FieldSerializer, LookupFieldSerializer
)
from geokey.categories.mixins import CategoryMixin, FieldMixin
from geokey.contributions.models import Observation


# #############################################################################
#
# ADMIN VIEWS
#
# #############################################################################

class CategoryList(LoginRequiredMixin, ProjectContext, TemplateView):
    """A list of all categories page."""

    template_name = 'categories/category_list.html'


class CategoryCreate(LoginRequiredMixin, ProjectContext, CreateView):
    """Create category page."""

    template_name = 'categories/category_create.html'
    form_class = CategoryCreateForm

    def get_context_data(self, **kwargs):
        """
        Return the context to render the view.

        Overwrite the method to add the project to the context.

        Returns
        -------
        dict
            Context.
        """
        return super(CategoryCreate, self).get_context_data(
            self.kwargs['project_id'],
            **kwargs
        )

    def form_valid(self, form):
        """
        Create a new category when form is valid.

        Parameters
        ----------
        form : geokey.categories.forms.CategoryCreateForm
            Represents the user input.
        """
        data = form.cleaned_data
        project = Project.objects.as_admin(
            self.request.user,
            self.kwargs['project_id']
        )

        if project:
            if project.islocked:
                messages.error(
                    self.request,
                    'The project is locked. New categories cannot be created.'
                )
                return redirect(
                    'admin:category_create',
                    project_id=self.kwargs['project_id']
                )
            else:
                category = Category.objects.create(
                    project=project,
                    creator=self.request.user,
                    name=strip_tags(data.get('name')),
                    description=strip_tags(data.get('description')),
                    default_status=data.get('default_status')
                )

                add_another_url = reverse(
                    'admin:category_create',
                    kwargs={'project_id': self.kwargs['project_id']}
                )
                messages.success(
                    self.request,
                    mark_safe('The category has been created. <a href="%s"> '
                              'Add another category.</a>' % add_another_url)
                )
                return redirect(
                    'admin:category_overview',
                    project_id=self.kwargs['project_id'],
                    category_id=category.id
                )

        return self.render_to_response(self.get_context_data(form=form))


class CategoryOverview(LoginRequiredMixin, CategoryMixin, TemplateView):
    """Category overview page."""

    template_name = 'categories/category_overview.html'


class CategorySettings(LoginRequiredMixin, CategoryMixin, TemplateView):
    """Category settings page."""

    template_name = 'categories/category_settings.html'

    def get_context_data(self, project_id, category_id):
        """
        Return the context to render the view.

        Overwrites the method to a number of contributions for the category and
        available status types to the context.

        Parameters
        ----------
        project_id : int
            Identifies the project in the database.
        category_id : int
            Identifies the category in the database.

        Returns
        -------
        dict
            Context.
        """
        context = super(CategorySettings, self).get_context_data(
            project_id,
            category_id,
            status_types=STATUS
        )
        category = context.get('category')

        if category:
            context['num_contributions'] = Observation.objects.filter(
                category=category
            ).count()

        return context

    def post(self, request, project_id, category_id):
        """
        Update category settings.

        Parameters
        ----------
        request : django.http.HttpRequest
            Object representing the request.
        project_id : int
            Identifies the project in the database.
        category_id : int
            Identifies the category in the database.

        Returns
        -------
        django.http.HttpResponse
            Rendered template.
        """
        data = request.POST
        context = self.get_context_data(project_id, category_id)
        category = context.get('category')

        if category:
            category.name = strip_tags(data.get('name'))
            category.description = strip_tags(data.get('description'))
            category.default_status = data.get('default_status')

            if category.fields.exists():
                try:
                    display_field = category.fields.get(
                        pk=data.get('display_field'))
                except:
                    display_field = None

                if category.display_field != display_field:
                    category.display_field = display_field
                    for observation in category.observation_set.all():
                        observation.update_display_field()
                        observation.save()

                try:
                    expiry_field = category.fields.get(
                        pk=data.get('expiry_field'))
                except:
                    expiry_field = None

                if (expiry_field and
                        expiry_field.type_name not in [
                        'Date and Time',
                        'Date']):
                    messages.error(
                        self.request,
                        'Only `Date and Time` and `Date` fields can be used.'
                    )
                elif category.expiry_field != expiry_field:
                    category.expiry_field = expiry_field
                    for observation in category.observation_set.all():
                        observation.update_expiry_field()
                        observation.save()

            category.save()
            messages.success(self.request, 'The category has been updated.')
            context['category'] = category

        return self.render_to_response(context)


class CategoryDisplay(LoginRequiredMixin, CategoryMixin, TemplateView):
    """Category display page."""

    template_name = 'categories/category_display.html'

    def post(self, request, project_id, category_id):
        """
        Update category display.

        Parameters
        ----------
        request : django.http.HttpRequest
            Object representing the request.
        project_id : int
            Identifies the project in the database.
        category_id : int
            Identifies the category in the database.

        Returns
        -------
        django.http.HttpResponse
            Rendered template.
        """
        data = request.POST
        context = self.get_context_data(project_id, category_id)
        category = context.get('category')

        if category:
            symbol = request.FILES.get('symbol')
            category.colour = data.get('colour')

            if symbol is not None:
                category.symbol.delete()
                category.symbol = symbol
            elif data.get('clear-symbol') == 'true':
                category.symbol.delete()
                category.symbol = None

            category.save()
            messages.success(
                self.request,
                'The display settings have been updated'
            )
            context['category'] = category

        return self.render_to_response(context)


class CategoryDelete(LoginRequiredMixin, CategoryMixin, TemplateView):
    """Delete category page."""

    template_name = 'base.html'

    def get(self, request, project_id, category_id):
        """
        Delete the category.

        Parameters
        ----------
        request : django.http.HttpRequest
            Object representing the request.
        project_id : int
            Identifies the project in the database.
        category_id : int
            Identifies the category in the database.

        Returns
        -------
        django.http.HttpResponseRedirect
            Redirects to category list if category is deleted, category
            settings if project is locked.
        django.http.HttpResponse
            Rendered template, if project or category does not exist.
        """
        context = self.get_context_data(project_id, category_id)
        category = context.get('category')

        if category:
            if category.project.islocked:
                messages.error(
                    self.request,
                    'The project is locked. Category cannot be deleted.'
                )
                return redirect(
                    'admin:category_settings',
                    project_id=project_id,
                    category_id=category_id
                )
            else:
                category.delete()
                messages.success(
                    self.request,
                    'The category has been deleted.'
                )
                return redirect('admin:category_list', project_id=project_id)

        return self.render_to_response(context)


class FieldCreate(LoginRequiredMixin, CategoryMixin, CreateView):
    """Creat field page."""

    template_name = 'categories/field_create.html'
    form_class = FieldCreateForm

    def get_context_data(self, data=None, **kwargs):
        """
        Return the context to render the view.

        Overwrite the method to add available field types the context.

        Parameters
        ----------
        project_id : int
            Identifies the project in the database.
        category_id : int
            Identifies the category in the database.

        Returns
        -------
        dict
            Context.
        """
        context = super(FieldCreate, self).get_context_data(
            self.kwargs['project_id'],
            self.kwargs['category_id'],
            **kwargs
        )
        context['fieldtypes'] = Field.get_field_types()
        return context

    def form_valid(self, form):
        """
        Create a new field when form is valid.

        Parameters
        ----------
        form : geokey.categories.forms.FieldCreateForm
            Represents the user input.
        """
        request = self.request
        data = form.cleaned_data
        category = Category.objects.as_admin(
            self.request.user,
            self.kwargs['project_id'],
            self.kwargs['category_id']
        )

        if category:
            if category.project.islocked:
                messages.error(
                    self.request,
                    'The project is locked. New fields cannot be created.'
                )
                return redirect(
                    'admin:category_field_create',
                    project_id=self.kwargs['project_id'],
                    category_id=self.kwargs['category_id']
                )
            else:
                proposed_key = slugify(strip_tags(data.get('name')))
                if len(proposed_key) < 1:
                    proposed_key = 'key'
                suggested_key = proposed_key

                count = 1
                while category.fields.filter(key=suggested_key).exists():
                    suggested_key = '%s-%s' % (proposed_key, count)
                    count = count + 1

                field = Field.create(
                    strip_tags(data.get('name')),
                    suggested_key,
                    strip_tags(data.get('description')),
                    data.get('required'),
                    category,
                    request.POST.get('type')
                )

                if isinstance(field, TextField):
                    field.textarea = request.POST.get('textarea') or False
                    field.maxlength = request.POST.get('maxlength') or None

                elif isinstance(field, NumericField):
                    field.minval = request.POST.get('minval') or None
                    field.maxval = request.POST.get('maxval') or None

                field.save()

                add_another_url = reverse(
                    'admin:category_field_create',
                    kwargs={
                        'project_id': self.kwargs['project_id'],
                        'category_id': self.kwargs['category_id']
                    }
                )
                messages.success(
                    self.request,
                    mark_safe('The field has been created. <a href="%s">Add '
                              'another field.</a>' % add_another_url)
                )
                return redirect(
                    'admin:category_field_settings',
                    project_id=self.kwargs['project_id'],
                    category_id=self.kwargs['category_id'],
                    field_id=field.id
                )

        return self.render_to_response(self.get_context_data(form=form))


class FieldSettings(LoginRequiredMixin, FieldMixin, TemplateView):
    """Field settings page."""

    template_name = 'categories/field_settings.html'

    def get_context_data(self, project_id, category_id, field_id, **kwargs):
        """
        Return the context to render the view.

        Overwrites the method to add available field types to the context.

        Parameters
        ----------
        project_id : int
            Identifies the project in the database.
        category_id : int
            Identifies the category in the database.
        field_id : int
            Identifies the field in the database.

        Returns
        -------
        dict
            Context
        """
        return super(FieldSettings, self).get_context_data(
            project_id,
            category_id,
            field_id,
            status_types=STATUS
        )

    def post(self, request, project_id, category_id, field_id):
        """
        Update field settings.

        Parameters
        ----------
        request : django.http.HttpRequest
            Object representing the request.
        project_id : int
            Identifies the project in the database.
        category_id : int
            Identifies the category in the database.
        field_id : int
            Identifies the field in the database.

        Returns
        -------
        django.http.HttpResponse
            Rendered template.
        """
        data = request.POST
        context = self.get_context_data(
            project_id,
            category_id,
            field_id
        )
        field = context.get('field')

        if field:
            field.name = strip_tags(data.get('name'))
            field.description = strip_tags(data.get('description'))
            field.required = data.get('required') or False

            if isinstance(field, TextField):
                field.textarea = data.get('textarea') or False
                field.maxlength = data.get('maxlength') or None
            elif isinstance(field, NumericField):
                field.minval = data.get('minval') or None
                field.maxval = data.get('maxval') or None

            field.save()
            messages.success(self.request, 'The field has been updated.')
            context['field'] = field

        return self.render_to_response(context)


class FieldDelete(LoginRequiredMixin, FieldMixin, TemplateView):
    """Delete field page."""

    template_name = 'base.html'

    def get(self, request, project_id, category_id, field_id):
        """
        Delete the field.

        Parameters
        ----------
        request : django.http.HttpRequest
            Object representing the request.
        project_id : int
            Identifies the project in the database.
        category_id : int
            Identifies the category in the database.
        field_id : int
            Identifies the field in the database.

        Returns
        -------
        django.http.HttpResponseRedirect
            Redirects to category overview if field is deleted, field
            settings if project is locked or field is set as display/expiry
            field for the category.
        django.http.HttpResponse
            Rendered template, if project, category or field does not exist.
        """
        context = self.get_context_data(project_id, category_id, field_id)
        field = context.get('field')

        if field:
            if field.category.project.islocked:
                messages.error(
                    self.request,
                    'The project is locked. Field cannot be deleted.'
                )
            elif context.get('is_display_field'):
                messages.error(
                    self.request,
                    'The field is set as display field and cannot be deleted.'
                )
            elif context.get('is_expiry_field'):
                messages.error(
                    self.request,
                    'The field is set as expiry field and cannot be deleted.'
                )
            else:
                field.delete()
                messages.success(self.request, 'The field has been deleted.')
                return redirect(
                    'admin:category_overview',
                    project_id=project_id,
                    category_id=category_id
                )

            return redirect(
                'admin:category_field_settings',
                project_id=project_id,
                category_id=category_id,
                field_id=field_id
            )

        return self.render_to_response(context)


# #############################################################################
#
# AJAX API
#
# #############################################################################

class CategoryUpdate(APIView):

    """
    API endpoints for a category in the AJAX API.
    """

    @handle_exceptions_for_ajax
    def get(self, request, project_id, category_id):
        """
        Handles the GET request.

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request
        project_id : int
            Identifies the project in the database
        category_id : int
            Identifies the category in the database

        Return
        ------
        rest_framework.response.Response
            Response to the request
        """

        category = Category.objects.as_admin(
            request.user, project_id, category_id)

        serializer = CategorySerializer(category)
        return Response(serializer.data)

    @handle_exceptions_for_ajax
    def put(self, request, project_id, category_id):
        """
        Handles the POST request and updates the category.

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request
        project_id : int
            Identifies the project in the database
        category_id : int
            Identifies the category in the database

        Return
        ------
        rest_framework.response.Response
            Response to the request
        """

        category = Category.objects.as_admin(
            request.user, project_id, category_id)

        serializer = CategorySerializer(
            category, data=request.data, partial=True,
            fields=('id', 'name', 'description', 'status'))

        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data)

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class FieldUpdate(APIView):

    """
    API endpoints for fields.
    """

    @handle_exceptions_for_ajax
    def put(self, request, project_id, category_id, field_id,
            format=None):
        """
        Handles the POST request and updates the category

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request
        project_id : int
            Identifies the project in the database
        category_id : int
            Identifies the category in the database
        field_id : int
            Identifies the field in the database

        Return
        ------
        rest_framework.response.Response
            Response to the request
        """

        field = Field.objects.as_admin(
            request.user, project_id, category_id, field_id)

        serializer = FieldSerializer(
            field, data=request.data, partial=True
        )

        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data)

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class FieldLookups(APIView):

    """
    API endpoint for lookupvalues.
    """

    @handle_exceptions_for_ajax
    def post(self, request, project_id, category_id, field_id,
             format=None):
        """
        Handles the POST request and adds a lookupvalue to the field.

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request
        project_id : int
            Identifies the project in the database
        category_id : int
            Identifies the category in the database
        field_id : int
            Identifies the field in the database

        Return
        ------
        rest_framework.response.Response
            Response to the request
        """

        field = Field.objects.as_admin(
            request.user, project_id, category_id, field_id)
        name = strip_tags(request.data.get('name'))

        if field.category.project.islocked:
            return Response(
                {'error': 'The project is locked.'},
                status=status.HTTP_400_BAD_REQUEST
            )

        if isinstance(field, LookupField):
            LookupValue.objects.create(name=name, field=field)

            serializer = LookupFieldSerializer(field)
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        elif isinstance(field, MultipleLookupField):
            MultipleLookupValue.objects.create(name=name, field=field)

            serializer = LookupFieldSerializer(field)
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        else:
            return Response(
                {'error': 'This field is not a lookup field.'},
                status=status.HTTP_404_NOT_FOUND
            )


class FieldLookupsUpdate(APIView):

    """
    API endpoint for lookupvalues.
    """

    def get_field(self, user, project_id, category_id, field_id):
        field = Field.objects.as_admin(
            user, project_id, category_id, field_id)

        if (isinstance(field, LookupField) or
                isinstance(field, MultipleLookupField)):
            return field
        else:
            return None

    @handle_exceptions_for_ajax
    def post(self, request, project_id, category_id, field_id, value_id):
        """
        Handles the POST request and updates the lookupvalue symbol.

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request
        project_id : int
            Identifies the project in the database
        category_id : int
            Identifies the category in the database
        field_id : int
            Identifies the field in the database
        value_id : int
            Identifies the lookupvalue in the database

        Return
        ------
        rest_framework.response.Response
            Response to the request
        """

        field = self.get_field(request.user, project_id, category_id, field_id)

        if field:
            if field.category.project.islocked:
                return Response(
                    {'error': 'The project is locked.'},
                    status=status.HTTP_400_BAD_REQUEST
                )
            else:
                value = field.lookupvalues.get(pk=value_id)

                name = request.data.get('name')

                if name:
                    value.name = strip_tags(name)

                symbol = request.FILES.get('symbol')

                if symbol:
                    value.symbol.delete()
                    value.symbol = symbol
                elif request.POST.get('clear-symbol') == 'true':
                    value.symbol.delete()
                    value.symbol = None

                value.save()

                return Response({
                    'id': value.id,
                    'name': value.name,
                    'symbol': value.symbol.url if value.symbol else None
                })
        else:
            return Response(
                {'error': 'This field is not a lookup field.'},
                status=status.HTTP_404_NOT_FOUND
            )

    @handle_exceptions_for_ajax
    def delete(self, request, project_id, category_id, field_id, value_id):
        """
        Handles the DELETE request and removes the lookupvalue the category.

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request
        project_id : int
            Identifies the project in the database
        category_id : int
            Identifies the category in the database
        field_id : int
            Identifies the field in the database
        value_id : int
            Identifies the lookupvalue in the database

        Return
        ------
        rest_framework.response.Response
            Response to the request
        """

        field = self.get_field(request.user, project_id, category_id, field_id)

        if field:
            if field.category.project.islocked:
                return Response(
                    {'error': 'The project is locked.'},
                    status=status.HTTP_400_BAD_REQUEST
                )
            else:
                field.lookupvalues.get(pk=value_id).delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
        else:
            return Response(
                {'error': 'This field is not a lookup field.'},
                status=status.HTTP_404_NOT_FOUND
            )


class FieldsReorderView(APIView):
    """Ajax API to reorder fields of a category."""

    @handle_exceptions_for_ajax
    def post(self, request, project_id, category_id):
        """
        Handle POST request.

        Reorder fields of the category.

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request.
        project_id : int
            Identifies the project in the database.
        category_id : int
            Identifies the category in the database.

        Returns
        -------
        rest_framework.response.Response
            Response to the request.
        """
        category = Category.objects.as_admin(
            request.user,
            project_id,
            category_id
        )

        try:
            category.reorder_fields(request.data.get('order'))
            serializer = CategorySerializer(category)
            return Response(serializer.data)
        except Field.DoesNotExist:
            return Response(
                {'error': 'One or more field IDs were not found.'},
                status=status.HTTP_400_BAD_REQUEST
            )


# ############################################################################
#
# PUBLIC API
#
# ############################################################################

class SingleCategory(APIView):
    """Public API for a single category."""

    @handle_exceptions_for_ajax
    def get(self, request, project_id, category_id):
        """
        Handle GET request.

        Return the complete category (including all fields).

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request.
        project_id : int
            Identifies the project in the database.
        category_id : int
            Identifies the category in the database.

        Returns
        -------
        rest_framework.response.Response
            Response to the request.
        """
        category = Category.objects.get_single(
            request.user,
            project_id,
            category_id
        )
        serializer = CategorySerializer(category)
        return Response(serializer.data)



"""Tests for logger: model Project."""

from django.test import TestCase
from django.contrib.gis.geos import GEOSGeometry

from geokey.core.models import LoggerHistory
from geokey.users.tests.model_factories import UserFactory
from geokey.projects.tests.model_factories import ProjectFactory, AdminsFactory


class LogProjectTest(TestCase):
    """Test model Project."""

    def setUp(self):
        """Set up test."""
        self.user = UserFactory.create()
        self.project = ProjectFactory.create(**{
            'creator': self.user})

    def test_log_create(self):
        """Test when project gets created."""
        log_count_init = LoggerHistory.objects.count()
        project = ProjectFactory.create(**{'creator': self.user})

        log_count = LoggerHistory.objects.count()
        self.assertEqual(log_count, log_count_init + 2)

        logs = LoggerHistory.objects.all().order_by('-pk')[:2]

        # Project gets created
        self.assertNotEqual(logs[1].user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(logs[1].project, {
            'id': str(project.id),
            'name': project.name})
        self.assertEqual(logs[1].usergroup, None)
        self.assertEqual(logs[1].category, None)
        self.assertEqual(logs[1].field, None)
        self.assertEqual(logs[1].location, None)
        self.assertEqual(logs[1].observation, None)
        self.assertEqual(logs[1].comment, None)
        self.assertEqual(logs[1].subset, None)
        self.assertEqual(logs[1].action, {
            'id': 'created',
            'class': 'Project'})
        self.assertEqual(logs[1].historical, None)

        # Project creator gets added as admin
        self.assertNotEqual(logs[0].user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(logs[0].project, {
            'id': str(project.id),
            'name': project.name})
        self.assertEqual(logs[0].usergroup, None)
        self.assertEqual(logs[0].category, None)
        self.assertEqual(logs[0].field, None)
        self.assertEqual(logs[0].location, None)
        self.assertEqual(logs[0].observation, None)
        self.assertEqual(logs[0].comment, None)
        self.assertEqual(logs[0].subset, None)
        self.assertEqual(logs[0].action, {
            'id': 'created',
            'class': 'Admins',
            'user_id': str(self.user.id),
            'user_display_name': self.user.display_name})
        self.assertEqual(logs[0].historical, None)

    def test_log_delete(self):
        """Test when project gets deleted."""
        project_id = self.project.id
        project_name = self.project.name
        log_count_init = LoggerHistory.objects.count()
        self.project.delete()

        log_count = LoggerHistory.objects.count()
        self.assertEqual(log_count, log_count_init + 2)

        logs = LoggerHistory.objects.all().order_by('-pk')[:2]

        # Project creator gets removed from admins
        self.assertNotEqual(logs[1].user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(logs[1].project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(logs[1].usergroup, None)
        self.assertEqual(logs[1].category, None)
        self.assertEqual(logs[1].field, None)
        self.assertEqual(logs[1].location, None)
        self.assertEqual(logs[1].observation, None)
        self.assertEqual(logs[1].comment, None)
        self.assertEqual(logs[1].subset, None)
        self.assertEqual(logs[1].action, {
            'id': 'deleted',
            'class': 'Admins',
            'user_id': str(self.user.id),
            'user_display_name': self.user.display_name})
        self.assertEqual(logs[1].historical, None)

        # Project gets deleted
        self.assertNotEqual(logs[0].user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(logs[0].project, {
            'id': str(project_id),
            'name': project_name})
        self.assertEqual(logs[0].usergroup, None)
        self.assertEqual(logs[0].category, None)
        self.assertEqual(logs[0].field, None)
        self.assertEqual(logs[0].location, None)
        self.assertEqual(logs[0].observation, None)
        self.assertEqual(logs[0].comment, None)
        self.assertEqual(logs[0].subset, None)
        self.assertEqual(logs[0].action, {
            'id': 'deleted',
            'class': 'Project',
            'field': 'status',
            'value': 'deleted'})
        history = self.project.history.get(pk=logs[0].historical.get('id'))
        self.assertEqual(history.id, project_id)
        self.assertEqual(history.name, project_name)

    def test_log_update_name(self):
        """Test when name changes."""
        log_count_init = LoggerHistory.objects.count()
        original_name = self.project.name
        self.project.name = '%s UPDATED' % self.project.name
        self.project.save()

        log = LoggerHistory.objects.last()
        log_count = LoggerHistory.objects.count()

        self.assertNotEqual(log.user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(log.project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(log.usergroup, None)
        self.assertEqual(log.category, None)
        self.assertEqual(log.field, None)
        self.assertEqual(log.location, None)
        self.assertEqual(log.observation, None)
        self.assertEqual(log.comment, None)
        self.assertEqual(log.subset, None)
        self.assertEqual(log.action, {
            'id': 'updated',
            'class': 'Project',
            'field': 'name'})
        self.assertEqual(log_count, log_count_init + 1)
        history = self.project.history.get(pk=log.historical.get('id'))
        self.assertEqual(history.id, self.project.id)
        self.assertEqual(history.name, original_name)

    def test_log_update_status(self):
        """Test when status changes."""
        log_count_init = LoggerHistory.objects.count()

        original_status = self.project.status
        self.project.status = 'inactive'
        self.project.save()

        log = LoggerHistory.objects.last()
        log_count = LoggerHistory.objects.count()

        self.assertNotEqual(log.user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(log.project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(log.usergroup, None)
        self.assertEqual(log.category, None)
        self.assertEqual(log.field, None)
        self.assertEqual(log.location, None)
        self.assertEqual(log.observation, None)
        self.assertEqual(log.comment, None)
        self.assertEqual(log.subset, None)
        self.assertEqual(log.action, {
            'id': 'updated',
            'class': 'Project',
            'field': 'status',
            'value': self.project.status})
        self.assertEqual(log_count, log_count_init + 1)
        history = self.project.history.get(pk=log.historical.get('id'))
        self.assertEqual(history.id, self.project.id)
        self.assertEqual(history.status, original_status)

        original_status = self.project.status
        self.project.status = 'active'
        self.project.save()

        log = LoggerHistory.objects.last()
        log_count = LoggerHistory.objects.count()

        self.assertNotEqual(log.user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(log.project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(log.usergroup, None)
        self.assertEqual(log.category, None)
        self.assertEqual(log.field, None)
        self.assertEqual(log.location, None)
        self.assertEqual(log.observation, None)
        self.assertEqual(log.comment, None)
        self.assertEqual(log.subset, None)
        self.assertEqual(log.action, {
            'id': 'updated',
            'class': 'Project',
            'field': 'status',
            'value': self.project.status})
        self.assertEqual(log_count, log_count_init + 2)
        history = self.project.history.get(pk=log.historical.get('id'))
        self.assertEqual(history.id, self.project.id)
        self.assertEqual(history.status, original_status)

    def test_log_update_isprivate(self):
        """Test when privacy changes."""
        log_count_init = LoggerHistory.objects.count()

        original_isprivate = self.project.isprivate
        self.project.isprivate = False
        self.project.save()

        log = LoggerHistory.objects.last()
        log_count = LoggerHistory.objects.count()

        self.assertNotEqual(log.user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(log.project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(log.usergroup, None)
        self.assertEqual(log.category, None)
        self.assertEqual(log.field, None)
        self.assertEqual(log.location, None)
        self.assertEqual(log.observation, None)
        self.assertEqual(log.comment, None)
        self.assertEqual(log.subset, None)
        self.assertEqual(log.action, {
            'id': 'updated',
            'class': 'Project',
            'field': 'isprivate',
            'value': str(self.project.isprivate)})
        self.assertEqual(log_count, log_count_init + 1)
        history = self.project.history.get(pk=log.historical.get('id'))
        self.assertEqual(history.id, self.project.id)
        self.assertEqual(history.isprivate, original_isprivate)

        original_isprivate = self.project.isprivate
        self.project.isprivate = True
        self.project.save()

        log = LoggerHistory.objects.last()
        log_count = LoggerHistory.objects.count()

        self.assertNotEqual(log.user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(log.project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(log.usergroup, None)
        self.assertEqual(log.category, None)
        self.assertEqual(log.field, None)
        self.assertEqual(log.location, None)
        self.assertEqual(log.observation, None)
        self.assertEqual(log.comment, None)
        self.assertEqual(log.subset, None)
        self.assertEqual(log.action, {
            'id': 'updated',
            'class': 'Project',
            'field': 'isprivate',
            'value': str(self.project.isprivate)})
        self.assertEqual(log_count, log_count_init + 2)
        history = self.project.history.get(pk=log.historical.get('id'))
        self.assertEqual(history.id, self.project.id)
        self.assertEqual(history.isprivate, original_isprivate)

    def test_log_update_islocked(self):
        """Test when locker changes."""
        log_count_init = LoggerHistory.objects.count()

        original_islocked = self.project.islocked
        self.project.islocked = True
        self.project.save()

        log = LoggerHistory.objects.last()
        log_count = LoggerHistory.objects.count()

        self.assertNotEqual(log.user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(log.project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(log.usergroup, None)
        self.assertEqual(log.category, None)
        self.assertEqual(log.field, None)
        self.assertEqual(log.location, None)
        self.assertEqual(log.observation, None)
        self.assertEqual(log.comment, None)
        self.assertEqual(log.subset, None)
        self.assertEqual(log.action, {
            'id': 'updated',
            'class': 'Project',
            'field': 'islocked',
            'value': str(self.project.islocked)})
        self.assertEqual(log_count, log_count_init + 1)
        history = self.project.history.get(pk=log.historical.get('id'))
        self.assertEqual(history.id, self.project.id)
        self.assertEqual(history.islocked, original_islocked)

        original_islocked = self.project.islocked
        self.project.islocked = False
        self.project.save()

        log = LoggerHistory.objects.last()
        log_count = LoggerHistory.objects.count()

        self.assertNotEqual(log.user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(log.project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(log.usergroup, None)
        self.assertEqual(log.category, None)
        self.assertEqual(log.field, None)
        self.assertEqual(log.location, None)
        self.assertEqual(log.observation, None)
        self.assertEqual(log.comment, None)
        self.assertEqual(log.subset, None)
        self.assertEqual(log.action, {
            'id': 'updated',
            'class': 'Project',
            'field': 'islocked',
            'value': str(self.project.islocked)})
        self.assertEqual(log_count, log_count_init + 2)
        history = self.project.history.get(pk=log.historical.get('id'))
        self.assertEqual(history.id, self.project.id)
        self.assertEqual(history.islocked, original_islocked)

    def test_log_update_contributing_permissions(self):
        """Test when contributing permissions changes."""
        log_count_init = LoggerHistory.objects.count()

        original_everyone_contributes = self.project.everyone_contributes
        self.project.everyone_contributes = 'auth'
        self.project.save()

        log = LoggerHistory.objects.last()
        log_count = LoggerHistory.objects.count()

        self.assertNotEqual(log.user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(log.project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(log.usergroup, None)
        self.assertEqual(log.category, None)
        self.assertEqual(log.field, None)
        self.assertEqual(log.location, None)
        self.assertEqual(log.observation, None)
        self.assertEqual(log.comment, None)
        self.assertEqual(log.subset, None)
        self.assertEqual(log.action, {
            'id': 'updated',
            'class': 'Project',
            'field': 'everyone_contributes',
            'value': self.project.everyone_contributes})
        self.assertEqual(log_count, log_count_init + 1)
        history = self.project.history.get(pk=log.historical.get('id'))
        self.assertEqual(history.id, self.project.id)
        self.assertEqual(
            history.everyone_contributes,
            original_everyone_contributes)

        original_everyone_contributes = self.project.everyone_contributes
        self.project.everyone_contributes = 'false'
        self.project.save()

        log = LoggerHistory.objects.last()
        log_count = LoggerHistory.objects.count()

        self.assertNotEqual(log.user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(log.project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(log.usergroup, None)
        self.assertEqual(log.category, None)
        self.assertEqual(log.field, None)
        self.assertEqual(log.location, None)
        self.assertEqual(log.observation, None)
        self.assertEqual(log.comment, None)
        self.assertEqual(log.subset, None)
        self.assertEqual(log.action, {
            'id': 'updated',
            'class': 'Project',
            'field': 'everyone_contributes',
            'value': self.project.everyone_contributes})
        self.assertEqual(log_count, log_count_init + 2)
        history = self.project.history.get(pk=log.historical.get('id'))
        self.assertEqual(history.id, self.project.id)
        self.assertEqual(
            history.everyone_contributes,
            original_everyone_contributes)

        original_everyone_contributes = self.project.everyone_contributes
        self.project.everyone_contributes = 'true'
        self.project.save()

        log = LoggerHistory.objects.last()
        log_count = LoggerHistory.objects.count()

        self.assertNotEqual(log.user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(log.project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(log.usergroup, None)
        self.assertEqual(log.category, None)
        self.assertEqual(log.field, None)
        self.assertEqual(log.location, None)
        self.assertEqual(log.observation, None)
        self.assertEqual(log.comment, None)
        self.assertEqual(log.subset, None)
        self.assertEqual(log.action, {
            'id': 'updated',
            'class': 'Project',
            'field': 'everyone_contributes',
            'value': self.project.everyone_contributes})
        self.assertEqual(log_count, log_count_init + 3)
        history = self.project.history.get(pk=log.historical.get('id'))
        self.assertEqual(history.id, self.project.id)
        self.assertEqual(
            history.everyone_contributes,
            original_everyone_contributes)

    def test_log_update_geographic_extent(self):
        """Test when geographic extent changes."""
        log_count_init = LoggerHistory.objects.count()
        original_geographic_extent = self.project.geographic_extent
        self.project.geographic_extent = GEOSGeometry(
            '{"type": "Polygon","coordinates":'
            '[[[-0.505,51.682],[-0.53,51.327],'
            '[0.225,51.323],[0.167,51.667],[-0.505,51.682]]]}')
        self.project.save()

        log = LoggerHistory.objects.last()
        log_count = LoggerHistory.objects.count()

        self.assertNotEqual(log.user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(log.project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(log.usergroup, None)
        self.assertEqual(log.category, None)
        self.assertEqual(log.field, None)
        self.assertEqual(log.location, None)
        self.assertEqual(log.observation, None)
        self.assertEqual(log.comment, None)
        self.assertEqual(log.subset, None)
        self.assertEqual(log.action, {
            'id': 'updated',
            'class': 'Project',
            'field': 'geographic_extent'})
        self.assertEqual(log_count, log_count_init + 1)
        history = self.project.history.get(pk=log.historical.get('id'))
        self.assertEqual(history.id, self.project.id)
        self.assertEqual(history.geographic_extent, original_geographic_extent)

    def test_log_update_multiple_fields(self):
        """Test when multiple model fields changes."""
        log_count_init = LoggerHistory.objects.count()
        original_isprivate = self.project.isprivate
        original_islocked = self.project.islocked
        self.project.isprivate = False
        self.project.islocked = True
        self.project.save()

        log_count = LoggerHistory.objects.count()
        self.assertEqual(log_count, log_count_init + 2)

        logs = LoggerHistory.objects.all().order_by('-pk')[:2]

        # 1st changed field
        self.assertNotEqual(logs[1].user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(logs[1].project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(logs[1].category, None)
        self.assertEqual(logs[1].field, None)
        self.assertEqual(logs[1].action, {
            'id': 'updated',
            'class': 'Project',
            'field': 'isprivate',
            'value': str(self.project.isprivate)})
        history_2 = self.project.history.get(pk=logs[1].historical.get('id'))
        self.assertEqual(history_2.id, self.project.id)
        self.assertEqual(history_2.isprivate, original_isprivate)
        self.assertEqual(history_2.islocked, original_islocked)

        # 2nd changed field
        self.assertNotEqual(logs[0].user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(logs[0].project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(logs[0].category, None)
        self.assertEqual(logs[0].field, None)
        self.assertEqual(logs[0].action, {
            'id': 'updated',
            'class': 'Project',
            'field': 'islocked',
            'value': str(self.project.islocked)})
        history_1 = self.project.history.get(pk=logs[0].historical.get('id'))
        self.assertEqual(history_1.id, self.project.id)
        self.assertEqual(history_1.isprivate, original_isprivate)
        self.assertEqual(history_1.islocked, original_islocked)

        # History entry is only one per save
        self.assertEqual(history_1, history_2)

    def test_log_add_admin(self):
        """Test when admin is added."""
        log_count_init = LoggerHistory.objects.count()
        new_admin = UserFactory.create()
        AdminsFactory.create(project=self.project, user=new_admin)

        log = LoggerHistory.objects.last()
        log_count = LoggerHistory.objects.count()

        self.assertNotEqual(log.user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(log.project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(log.usergroup, None)
        self.assertEqual(log.category, None)
        self.assertEqual(log.field, None)
        self.assertEqual(log.location, None)
        self.assertEqual(log.observation, None)
        self.assertEqual(log.comment, None)
        self.assertEqual(log.subset, None)
        self.assertEqual(log.action, {
            'id': 'created',
            'class': 'Admins',
            'user_id': str(new_admin.id),
            'user_display_name': new_admin.display_name})
        self.assertEqual(log_count, log_count_init + 1)
        self.assertEqual(log.historical, None)

    def test_log_remove_admin(self):
        """Test when admin is removed."""
        existing_admin = UserFactory.create()
        admins_relation = AdminsFactory.create(
            project=self.project,
            user=existing_admin)
        log_count_init = LoggerHistory.objects.count()
        admins_relation.delete()

        log = LoggerHistory.objects.last()
        log_count = LoggerHistory.objects.count()

        self.assertNotEqual(log.user, {
            'id': str(self.user.id),
            'display_name': self.user.display_name})
        self.assertEqual(log.project, {
            'id': str(self.project.id),
            'name': self.project.name})
        self.assertEqual(log.usergroup, None)
        self.assertEqual(log.category, None)
        self.assertEqual(log.field, None)
        self.assertEqual(log.location, None)
        self.assertEqual(log.observation, None)
        self.assertEqual(log.comment, None)
        self.assertEqual(log.subset, None)
        self.assertEqual(log.action, {
            'id': 'deleted',
            'class': 'Admins',
            'user_id': str(existing_admin.id),
            'user_display_name': existing_admin.display_name})
        self.assertEqual(log_count, log_count_init + 1)
        self.assertEqual(log.historical, None)



"""Views for locations of contributions."""

from django.db.models import Q

from rest_framework import status
from rest_framework.views import APIView
from rest_framework.response import Response

from geokey.core.decorators import handle_exceptions_for_ajax
from geokey.users.models import User

from ..models import Location
from ..serializers import LocationSerializer


class LocationAbstractAPIView(APIView):
    """Abstract class for locations."""

    def get_user(self, request):
        """
        Get user of a request.

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request.

        Returns
        -------
        geokey.users.models.User
            User of a request.
        """
        user = request.user

        if user.is_anonymous():
            user = User.objects.get(display_name='AnonymousUser')

        return user


class LocationsAPIView(LocationAbstractAPIView):
    """Public API for all locations."""

    @handle_exceptions_for_ajax
    def get(self, request, project_id):
        """
        Handle GET request.

        Return a list of all locations of the project, that can be used for
        contributions.

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request.
        project_id : int
            Identifies the project in the database.

        Returns
        -------
        rest_framework.response.Respone
            Contains the serialised locations.
        """
        query = request.GET.get('query')
        locations = Location.objects.get_list(
            self.get_user(request),
            project_id
        )

        if query:
            query = query.lower()
            locations = locations.filter(
                Q(name__icontains=query) | Q(description__icontains=query)
            )

        serializer = LocationSerializer(locations, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)


class SingleLocationAPIView(LocationAbstractAPIView):
    """Public API for a single location."""

    @handle_exceptions_for_ajax
    def patch(self, request, project_id, location_id):
        """
        Handle GET request.

        Update the location.

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request.
        project_id : int
            Identifies the project in the database.
        location_id : int
            Identifies the location in the database.

        Returns
        -------
        rest_framework.response.Respone
            Contains the serialized location.
        """
        location = Location.objects.get_single(
            self.get_user(request),
            project_id,
            location_id
        )
        serializer = LocationSerializer(
            location,
            data=request.data,
            partial=True
        )

        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data)

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)



"""Tests for template tags of projects."""

import datetime

from django.test import TestCase
from ..templatetags import count, project_attributes

from geokey.users.models import User
from geokey.projects.models import Project


class CountTest(TestCase):
    def test_more_link_text(self):
        self.assertEqual(
            count.more_link_text(6, 'category', 'categories'),
            'Show 1 more category'
        )

        self.assertEqual(
            count.more_link_text(7, 'category', 'categories'),
            'Show 2 more categories'
        )

        self.assertEqual(
            count.more_link_text(5, 'category', 'categories', 4),
            'Show 1 more category'
        )

        self.assertEqual(
            count.more_link_text(6, 'category', 'categories', 4),
            'Show 2 more categories'
        )


class ProjectAttributesTest(TestCase):
    def test_project_attributes(self):
        user = User(display_name='Test Name')
        project = Project(
            isprivate=True,
            islocked=False,
            creator=user,
            created_at=datetime.datetime.now(),
            status='active'
        )
        html = project_attributes.project_attributes(project)

        self.assertIn(
            ('Created by %s on %s') % (
                user.display_name,
                project.created_at.strftime("%d %B %Y, %H:%M")
            ),
            html
        )
        self.assertNotIn(
            'Locked',
            html)
        self.assertIn(
            '<span class="label label-primary">Private</span>',
            html)
        self.assertNotIn(
            'Archived',
            html)

        project = Project(
            isprivate=False,
            islocked=True,
            creator=user,
            created_at=datetime.datetime.now(),
            status='inactive'
        )
        html = project_attributes.project_attributes(project)

        self.assertIn(
            ('Created by %s on %s') % (
                user.display_name,
                project.created_at.strftime("%d %B %Y, %H:%M")
            ),
            html
        )
        self.assertIn(
            '<span class="label label-primary">Public</span>',
            html)
        self.assertIn(
            '<span class="label label-warning">Archived</span>',
            html)



"""Core URLs."""

from django.conf import settings
from django.conf.urls import include, url
from django.conf.urls.static import static
from django.views.generic.base import RedirectView


urlpatterns = [
    url(
        r'^oauth2/',
        include('oauth2_provider.urls', namespace='oauth2_provider')
    ),
    url(
        r'^admin/',
        include('geokey.core.url.admin', namespace='admin')
    ),
    url(
        r'^admin/account/',
        include('allauth.urls')
    ),
    url(
        r'^ajax/',
        include('geokey.core.url.ajax', namespace='ajax')
    ),
    url(
        r'^api/',
        include('geokey.core.url.api', namespace='api')
    ),
    url(
        r'^',
        include('geokey.extensions.urls')
    ),
    url(
        r'^$',
        RedirectView.as_view(url='/admin/', permanent=True)
    ),
] + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)


if settings.DEBUG and settings.DEBUG_TOOLBAR:
    import debug_toolbar
    urlpatterns += [
        url(r'^debug-toolbar/', include(debug_toolbar.urls)),
    ]



"""Base settings."""

from os.path import abspath, dirname, join, normpath
from django.contrib import messages

# The ID of the current site in the django_site database table
# see: https://docs.djangoproject.com/en/1.8/ref/settings/#site-id
SITE_ID = 1

# Ensures that a trailing slash is always present in URLs
# see: https://docs.djangoproject.com/en/1.8/ref/settings/#std:setting-APPEND_SLASH
APPEND_SLASH = True

# Sets the default time zone
TIME_ZONE = 'UTC'

# Disbles Django's translation engine
# see: https://docs.djangoproject.com/en/1.8/ref/settings/#use-i18n
USE_I18N = False

# Enables localised date formatting
# see: https://docs.djangoproject.com/en/1.8/ref/settings/#use-l10n
USE_L10N = True

# A boolean that specifies if datetimes will be timezone-aware by default or not.
# see: A boolean that specifies if datetimes will be timezone-aware by default or not.
USE_TZ = True

# All Django applications installed. Includes Django modules, third-party
# packages and GeoKey modules. New third-party packages and GeoKey modules
# should be added here.
# see: https://docs.djangoproject.com/en/1.8/ref/settings/#installed-apps
INSTALLED_APPS = (
    # Django apps
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.gis',
    'django.contrib.sites',

    # Third-party apps
    'django_hstore',
    'oauth2_provider',
    'easy_thumbnails',
    'allauth',
    'allauth.account',
    'allauth.socialaccount',
    'rest_framework',
    'rest_framework_gis',

    # GeoKey apps
    'geokey.core',
    'geokey.projects',
    'geokey.categories',
    'geokey.contributions',
    'geokey.users',
    'geokey.applications',
    'geokey.superusertools',
    'geokey.extensions',
    'geokey.subsets',
)

# Middleware that is used with GeoKey to process HTTP requests and responses.
# see: https://docs.djangoproject.com/en/1.8/ref/settings/#std:setting-MIDDLEWARE_CLASSES
# Learn about Middleware: https://docs.djangoproject.com/en/1.8/topics/http/middleware/
MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'geokey.core.middleware.XsSharing',
    'geokey.core.middleware.RequestProvider',
)

# Settings for django-oauth-toolkit
# see: https://django-oauth-toolkit.readthedocs.org/en/latest/settings.html
OAUTH2_PROVIDER = {
    'SCOPES': {
        'read': 'Access private projects, contributions and other information',
        'write': 'Manage contributions, comments and media files'
    },
    'CLIENT_SECRET_GENERATOR_LENGTH': 40
}
OAUTH2_PROVIDER_APPLICATION_MODEL = 'applications.Application'

# Settings for Django REST Framework
# see: http://www.django-rest-framework.org/api-guide/settings/
REST_FRAMEWORK = {
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'rest_framework.authentication.SessionAuthentication',
        'oauth2_provider.ext.rest_framework.OAuth2Authentication',
    ),
}

# Avaiable message tags; for use with Django's messages Framework
# see: https://docs.djangoproject.com/en/1.8/ref/settings/#message-tags
MESSAGE_TAGS = {
    messages.DEBUG: 'debug',
    messages.INFO: 'info',
    messages.SUCCESS: 'success',
    messages.WARNING: 'warning',
    messages.ERROR: 'danger'
}

# Settings for django.contrib.auth, used for user authentication
# see: https://docs.djangoproject.com/en/1.8/ref/settings/#auth
AUTHENTICATION_BACKENDS = (
    # Needed to login by username in Django admin, regardless of `allauth`
    'django.contrib.auth.backends.ModelBackend',
    # `allauth` specific authentication methods, such as login by e-mail
    'allauth.account.auth_backends.AuthenticationBackend'
)
AUTH_USER_MODEL = 'users.User'
LOGIN_REDIRECT_URL = '/admin/dashboard/'
LOGIN_URL = '/admin/account/login/'


# django-allauth settings
# see: http://django-allauth.readthedocs.org/en/latest/configuration.html
ACCOUNT_ADAPTER = 'geokey.core.adapters.AccountAdapter'
ACCOUNT_LOGOUT_REDIRECT_URL = '/admin/account/login/'
ACCOUNT_USER_MODEL_USERNAME_FIELD = 'display_name'
ACCOUNT_EMAIL_REQUIRED = True
ACCOUNT_USERNAME_REQUIRED = True
ACCOUNT_AUTHENTICATION_METHOD = 'email'
ACCOUNT_LOGOUT_ON_GET = True
ACCOUNT_CONFIRM_EMAIL_ON_GET = True
ACCOUNT_EMAIL_CONFIRMATION_ANONYMOUS_REDIRECT_URL = '/admin/account/login/'
ACCOUNT_EMAIL_CONFIRMATION_AUTHENTICATED_REDIRECT_URL = '/admin/dashboard/'
ACCOUNT_FORMS = {
    'signup': 'geokey.users.forms.UserRegistrationForm',
    'change_password': 'geokey.users.forms.CustomPasswordChangeForm',
    'reset_password_from_key': 'geokey.users.forms.CustomResetPasswordKeyForm'
}
SOCIALACCOUNT_ADAPTER = 'geokey.core.adapters.SocialAccountAdapter'
SOCIALACCOUNT_QUERY_EMAIL = True
SOCIALACCOUNT_PROVIDERS = {}

SITE_ROOT = dirname(dirname(dirname(abspath(__file__))))
STATICFILES_DIRS = [normpath(join(SITE_ROOT, 'static'))]

ROOT_URLCONF = 'geokey.core.urls'

# Settings for Django's template engine
# see: https://docs.djangoproject.com/en/1.8/ref/settings/#std:setting-TEMPLATES
TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [normpath(join(SITE_ROOT, 'templates'))],
        'OPTIONS': {
            'loaders': [
                'django.template.loaders.filesystem.Loader',
                'django.template.loaders.app_directories.Loader',
            ],
            'context_processors': [
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'geokey.core.context_processors.project_settings',
                'django.contrib.messages.context_processors.messages'
            ],
        },
    },
]

# Custom GeoKey settings, enables video upload. Disabled by default, can be
# endabled by overwriting in local settings
ENABLE_VIDEO = False



"""Core image helpers."""

from PIL import Image
from StringIO import StringIO

from django.core.files.base import ContentFile


def get_image(file_name='test.png', width=200, height=200):
    image_file = StringIO()
    image = Image.new('RGBA', size=(width, height), color=(255, 0, 255))
    image.save(image_file, 'png')
    image_file.seek(0)

    the_file = ContentFile(image_file.read(), file_name)
    the_file.content_type = 'image/png'

    return the_file



"""Tests for views of subsets."""

import json

from django.test import TestCase
from django.http import HttpRequest
from django.core.urlresolvers import reverse
from django.template.loader import render_to_string
from django.contrib.auth.models import AnonymousUser
from django.contrib.sites.shortcuts import get_current_site
from django.contrib.messages import get_messages
from django.contrib.messages.storage.fallback import FallbackStorage

from geokey import version
from geokey.core.tests.helpers import render_helpers
from geokey.users.tests.model_factories import UserFactory
from geokey.projects.tests.model_factories import ProjectFactory
from geokey.categories.tests.model_factories import CategoryFactory

from .model_factories import SubsetFactory
from ..models import Subset
from ..views import (
    SubsetList,
    SubsetCreate,
    SubsetSettings,
    SubsetData,
    SubsetDelete
)


class SubsetListTest(TestCase):

    def setUp(self):

        self.view = SubsetList.as_view()
        self.request = HttpRequest()
        self.request.method = 'GET'
        self.request.user = AnonymousUser()

        setattr(self.request, 'session', 'session')
        messages = FallbackStorage(self.request)
        setattr(self.request, '_messages', messages)

    def test_get_with_anonymous(self):
        """
        Accessing the view with AnonymousUser should redirect to the login
        page
        """

        response = self.view(self.request)

        self.assertEqual(response.status_code, 302)
        self.assertIn('/admin/account/login/', response['location'])

    def test_get_with_user(self):
        """
        Accessing the view with normal user should render the page with an
        error message
        """

        user = UserFactory.create()
        project = ProjectFactory.create()

        self.request.user = user
        response = self.view(self.request, project_id=project.id).render()

        rendered = render_to_string(
            'subsets/subset_list.html',
            {
                'error_description': 'Project matching query does not exist.',
                'error': 'Not found.',
                'user': user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)

    def test_get_with_admin(self):
        """
        Accessing the view with project admin should render the page
        """

        project = ProjectFactory.create()
        user = project.creator

        self.request.user = user
        response = self.view(self.request, project_id=project.id).render()

        rendered = render_to_string(
            'subsets/subset_list.html',
            {
                'project': project,
                'user': user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)


class SubsetCreateTest(TestCase):

    def setUp(self):

        self.view = SubsetCreate.as_view()
        self.request = HttpRequest()
        self.request.method = 'GET'
        self.request.user = AnonymousUser()

        setattr(self.request, 'session', 'session')
        messages = FallbackStorage(self.request)
        setattr(self.request, '_messages', messages)

    def test_get_with_anonymous(self):
        """
        Accessing the view with AnonymousUser should redirect to the login
        page
        """

        response = self.view(self.request)

        self.assertEqual(response.status_code, 302)
        self.assertIn('/admin/account/login/', response['location'])

    def test_get_with_user(self):
        """
        Accessing the view with normal user should render the page with an
        error message
        """

        user = UserFactory.create()
        project = ProjectFactory.create()

        self.request.user = user
        response = self.view(self.request, project_id=project.id).render()

        rendered = render_to_string(
            'subsets/subset_create.html',
            {
                'error_description': 'Project matching query does not exist.',
                'error': 'Not found.',
                'user': user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)

    def test_get_with_admin(self):
        """
        Accessing the view with project admin should render the page
        """

        project = ProjectFactory.create()
        user = project.creator

        self.request.user = user
        response = self.view(self.request, project_id=project.id).render()

        rendered = render_to_string(
            'subsets/subset_create.html',
            {
                'project': project,
                'user': user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        response = render_helpers.remove_csrf(response.content.decode('utf-8'))
        self.assertEqual(response, rendered)

    def test_post_with_anonymous(self):
        """
        Updating with AnonymousUser should redirect to the login page
        """

        self.request.method = 'POST'
        self.request.POST = {
            'name': 'Name',
            'description': 'Description',
        }
        response = self.view(self.request)

        self.assertEqual(response.status_code, 302)
        self.assertIn('/admin/account/login/', response['location'])
        self.assertEqual(0, Subset.objects.count())

    def test_post_with_user(self):
        """
        Updating with normal user should render the page with an error message
        """

        self.request.method = 'POST'
        self.request.POST = {
            'name': 'Name',
            'description': 'Description',
        }

        project = ProjectFactory.create()
        CategoryFactory.create(project=project)
        user = UserFactory.create()

        self.request.user = user
        response = self.view(self.request, project_id=project.id).render()

        rendered = render_to_string(
            'subsets/subset_create.html',
            {
                'error_description': 'Project matching query does not exist.',
                'error': 'Not found.',
                'user': user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)
        self.assertEqual(0, Subset.objects.count())

    def test_post_with_admin(self):
        """
        Updating with project admin should create the subset and redirect to
        the subset data page
        """

        self.request.method = 'POST'
        self.request.POST = {
            'name': 'Name',
            'description': 'Description',
        }

        project = ProjectFactory.create()
        CategoryFactory.create(project=project)
        user = project.creator

        self.request.user = user
        response = self.view(self.request, project_id=project.id)

        self.assertEqual(1, Subset.objects.count())
        subset = Subset.objects.first()
        self.assertEqual(subset.name, 'Name')
        self.assertEqual(subset.description, 'Description')
        self.assertEqual(subset.project, project)
        self.assertEqual(subset.creator, user)

        self.assertEqual(response.status_code, 302)
        self.assertIn(
            '/admin/projects/%s/subsets/%s/' % (project.id, subset.id),
            response['location']
        )

    def test_post_on_locked_project_with_admin(self):
        """
        Updating with project admin when the project is locked should redirect
        to subset list page
        """

        self.request.method = 'POST'
        self.request.POST = {
            'name': 'Name',
            'description': 'Description',
        }

        project = ProjectFactory.create(islocked=True)
        CategoryFactory.create(project=project)
        user = project.creator

        self.request.user = user
        response = self.view(self.request, project_id=project.id)

        self.assertEqual(0, Subset.objects.count())
        self.assertEqual(response.status_code, 302)
        self.assertIn(
            '/admin/projects/%s/subsets/new/' % (project.id),
            response['location']
        )

    def test_post_on_project_when_no_categories_with_admin(self):
        """
        Updating with project admin when the project has no categories should
        redirect to subset list page
        """

        self.request.method = 'POST'
        self.request.POST = {
            'name': 'Name',
            'description': 'Description',
        }

        project = ProjectFactory.create(islocked=True)
        user = project.creator

        self.request.user = user
        response = self.view(self.request, project_id=project.id)

        self.assertEqual(0, Subset.objects.count())
        self.assertEqual(response.status_code, 302)
        self.assertIn(
            '/admin/projects/%s/subsets/new/' % (project.id),
            response['location']
        )


class SubsetSettingsTest(TestCase):

    def setUp(self):

        self.view = SubsetSettings.as_view()
        self.request = HttpRequest()
        self.request.method = 'GET'
        self.request.user = AnonymousUser()

        setattr(self.request, 'session', 'session')
        messages = FallbackStorage(self.request)
        setattr(self.request, '_messages', messages)

    def test_get_with_anonymous(self):
        """
        Accessing the view with AnonymousUser should redirect to the login
        page
        """

        subset = SubsetFactory.create()
        response = self.view(
            self.request,
            project_id=subset.project.id,
            subset_id=subset.id)

        self.assertEqual(response.status_code, 302)
        self.assertIn('/admin/account/login/', response['location'])

    def test_get_with_user(self):
        """
        Accessing the view with normal user should render the page with an
        error message
        """

        user = UserFactory.create()
        subset = SubsetFactory.create()

        self.request.user = user
        response = self.view(
            self.request,
            project_id=subset.project.id,
            subset_id=subset.id).render()

        rendered = render_to_string(
            'subsets/subset_settings.html',
            {
                'error_description': 'Project matching query does not exist.',
                'error': 'Not found.',
                'user': user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        response = render_helpers.remove_csrf(response.content.decode('utf-8'))
        self.assertEqual(response, rendered)

    def test_get_with_admin(self):
        """
        Accessing the view with project admin should render the page
        """

        subset = SubsetFactory.create()
        user = subset.project.creator

        self.request.user = user
        response = self.view(
            self.request,
            project_id=subset.project.id,
            subset_id=subset.id).render()

        rendered = render_to_string(
            'subsets/subset_settings.html',
            {
                'project': subset.project,
                'subset': subset,
                'user': user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        response = render_helpers.remove_csrf(response.content.decode('utf-8'))
        self.assertEqual(response, rendered)

    def test_get_non_existing_with_admin(self):
        """
        Accessing the view with project admin when subset does not exist should
        render the page with an error message
        """

        project = ProjectFactory.create()
        user = project.creator

        self.request.user = user
        response = self.view(
            self.request,
            project_id=project.id,
            subset_id=634842156456).render()

        rendered = render_to_string(
            'subsets/subset_settings.html',
            {
                'error_description': 'Subset matching query does not exist.',
                'error': 'Not found.',
                'user': user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)

    def test_post_with_anonymous(self):
        """
        Updating with AnonymousUser should redirect to the login page
        """

        subset = SubsetFactory.create()
        self.request.method = 'POST'
        self.request.POST = {
            'name': 'Name',
            'description': 'Description',
        }
        response = self.view(
            self.request,
            project_id=subset.project.id,
            subset_id=subset.id)

        self.assertEqual(response.status_code, 302)
        self.assertIn('/admin/account/login/', response['location'])

        reference = Subset.objects.get(pk=subset.id)
        self.assertNotEqual(reference.name, 'Name')
        self.assertNotEqual(reference.description, 'Description')

    def test_post_with_user(self):
        """
        Updating with normal user should render the page with an error message
        """

        user = UserFactory.create()
        subset = SubsetFactory.create()

        self.request.user = user
        self.request.method = 'POST'
        self.request.POST = {
            'name': 'Name',
            'description': 'Description',
        }

        response = self.view(
            self.request,
            project_id=subset.project.id,
            subset_id=subset.id).render()

        rendered = render_to_string(
            'subsets/subset_settings.html',
            {
                'error_description': 'Project matching query does not exist.',
                'error': 'Not found.',
                'user': user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)

        reference = Subset.objects.get(pk=subset.id)
        self.assertNotEqual(reference.name, 'Name')
        self.assertNotEqual(reference.description, 'Description')

    def test_post_with_admin(self):
        """
        Updating with project admin should render the page with a success
        message
        """

        subset = SubsetFactory.create()
        user = subset.project.creator

        self.request.user = user
        self.request.method = 'POST'
        self.request.POST = {
            'name': 'Name',
            'description': 'Description',
        }

        response = self.view(
            self.request,
            project_id=subset.project.id,
            subset_id=subset.id).render()

        reference = Subset.objects.get(pk=subset.id)
        self.assertEqual(reference.name, 'Name')
        self.assertEqual(reference.description, 'Description')

        rendered = render_to_string(
            'subsets/subset_settings.html',
            {
                'project': reference.project,
                'subset': reference,
                'user': user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version(),
                'messages': get_messages(self.request)
            }
        )
        self.assertEqual(response.status_code, 200)
        response = render_helpers.remove_csrf(response.content.decode('utf-8'))
        self.assertEqual(response, rendered)

    def test_post_non_existing_with_admin(self):
        """
        Update the view with project admin when subset does not exist should
        render the page withh an error message
        """

        project = ProjectFactory.create()
        user = project.creator

        self.request.user = user
        self.request.method = 'POST'
        self.request.POST = {
            'name': 'Name',
            'description': 'Description',
        }

        response = self.view(
            self.request,
            project_id=project.id,
            subset_id=634842156456).render()

        rendered = render_to_string(
            'subsets/subset_settings.html',
            {
                'error_description': 'Subset matching query does not exist.',
                'error': 'Not found.',
                'user': user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)


class SubsetDataTest(TestCase):

    def setUp(self):

        self.view = SubsetData.as_view()
        self.request = HttpRequest()
        self.request.method = 'GET'
        self.request.user = AnonymousUser()

        setattr(self.request, 'session', 'session')
        messages = FallbackStorage(self.request)
        setattr(self.request, '_messages', messages)

    def test_get_with_anonymous(self):
        """
        Accessing the view with AnonymousUser should redirect to the login
        page
        """

        subset = SubsetFactory.create()
        response = self.view(
            self.request,
            project_id=subset.project.id,
            subset_id=subset.id)

        self.assertEqual(response.status_code, 302)
        self.assertIn('/admin/account/login/', response['location'])

    def test_get_with_user(self):
        """
        Accessing the view with normal user should render the page with an
        error message
        """

        subset = SubsetFactory.create()
        user = UserFactory.create()

        self.request.user = user
        response = self.view(
            self.request,
            project_id=subset.project.id,
            subset_id=subset.id).render()

        rendered = render_to_string(
            'subsets/subset_data.html',
            {
                'error_description': 'Project matching query does not exist.',
                'error': 'Not found.',
                'user': user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)

    def test_get_with_admin(self):
        """
        Accessing the view with project admin should render the page
        """

        subset = SubsetFactory.create()
        user = subset.project.creator

        self.request.user = user
        response = self.view(
            self.request,
            project_id=subset.project.id,
            subset_id=subset.id).render()

        rendered = render_to_string(
            'subsets/subset_data.html',
            {
                'project': subset.project,
                'subset': subset,
                'user': user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)

    def test_post_with_anonymous(self):
        """
        Updating with AnonymousUser should redirect to the login page
        """

        subset = SubsetFactory.create()
        category = CategoryFactory.create(**{'project': subset.project})

        self.request.method = 'POST'
        self.request.POST = {
            'filters': '{ "%s": { } }' % category.id
        }

        response = self.view(
            self.request,
            project_id=subset.project.id,
            subset_id=subset.id)

        reference_subset = Subset.objects.get(pk=subset.id)
        self.assertIsNone(reference_subset.filters)

        self.assertEqual(response.status_code, 302)
        self.assertIn('/admin/account/login/', response['location'])

    def test_post_with_user(self):
        """
        Updating with normal user should render the page with an error message
        """

        subset = SubsetFactory.create()
        category = CategoryFactory.create(**{'project': subset.project})
        user = UserFactory.create()

        self.request.user = user
        self.request.method = 'POST'
        self.request.POST = {
            'filters': '{ "%s": { } }' % category.id
        }
        response = self.view(
            self.request,
            project_id=subset.project.id,
            subset_id=subset.id).render()

        reference_subset = Subset.objects.get(pk=subset.id)
        self.assertIsNone(reference_subset.filters)

        rendered = render_to_string(
            'subsets/subset_data.html',
            {
                'error_description': 'Project matching query does not exist.',
                'error': 'Not found.',
                'user': user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)

    def test_post_with_admin(self):
        """
        Updating with project admin should render the page with a success
        message
        """

        subset = SubsetFactory.create()
        category = CategoryFactory.create(**{'project': subset.project})
        user = subset.project.creator

        self.request.user = user
        self.request.method = 'POST'
        self.request.POST = {
            'filters': '{ "%s": { } }' % category.id
        }
        response = self.view(
            self.request,
            project_id=subset.project.id,
            subset_id=subset.id).render()

        reference = Subset.objects.get(pk=subset.id)
        self.assertEqual(
            reference.filters,
            json.loads('{ "%s": { } }' % category.id)
        )

        rendered = render_to_string(
            'subsets/subset_data.html',
            {
                'project': reference.project,
                'subset': reference,
                'user': user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'messages': get_messages(self.request),
                'GEOKEY_VERSION': version.get_version()
            }
        )

        self.assertEqual(response.status_code, 200)
        response = render_helpers.remove_csrf(response.content.decode('utf-8'))
        self.assertEqual(response, rendered)

    def test_post_non_existing_with_admin(self):
        """
        Updating with project admin when subset does not exist should render
        the page withh an error message
        """

        project = ProjectFactory.create()
        category = CategoryFactory.create(**{'project': project})
        user = project.creator

        self.request.user = user
        self.request.method = 'POST'
        self.request.POST = {
            'filters': '{ "%s": { } }' % category.id
        }
        response = self.view(
            self.request,
            project_id=project.id,
            subset_id=634842156456).render()

        rendered = render_to_string(
            'subsets/subset_data.html',
            {
                'error_description': 'Subset matching query does not exist.',
                'error': 'Not found.',
                'user': user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)

    def test_post_on_locked_project_with_admin(self):
        """
        Updating with project admin when the project is locked should render
        the page withh an error message
        """

        subset = SubsetFactory.create()
        subset.project.islocked = True
        subset.project.save()

        category = CategoryFactory.create(**{'project': subset.project})
        user = subset.project.creator

        self.request.user = user
        self.request.method = 'POST'
        self.request.POST = {
            'filters': '{ "%s": { } }' % category.id
        }
        response = self.view(
            self.request,
            project_id=subset.project.id,
            subset_id=subset.id).render()

        reference = Subset.objects.get(pk=subset.id)
        self.assertNotEqual(
            reference.filters,
            json.loads('{ "%s": { } }' % category.id)
        )

        rendered = render_to_string(
            'subsets/subset_data.html',
            {
                'project': reference.project,
                'subset': reference,
                'user': user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'messages': get_messages(self.request),
                'GEOKEY_VERSION': version.get_version()
            }
        )

        self.assertEqual(response.status_code, 200)
        response = render_helpers.remove_csrf(response.content.decode('utf-8'))
        self.assertEqual(response, rendered)

    def test_post_on_project_when_no_categories_with_admin(self):
        """
        Updating with project admin when the project has no categories should
        render the page withh an error message
        """

        subset = SubsetFactory.create()
        subset.project.islocked = True
        subset.project.save()

        user = subset.project.creator

        self.request.user = user
        self.request.method = 'POST'
        self.request.POST = {
            'filters': '{ "%s": { } }' % 154515
        }
        response = self.view(
            self.request,
            project_id=subset.project.id,
            subset_id=subset.id).render()

        reference = Subset.objects.get(pk=subset.id)
        self.assertNotEqual(
            reference.filters,
            json.loads('{ "%s": { } }' % 154515)
        )

        rendered = render_to_string(
            'subsets/subset_data.html',
            {
                'project': reference.project,
                'subset': reference,
                'user': user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'messages': get_messages(self.request),
                'GEOKEY_VERSION': version.get_version()
            }
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)


class SubsetDeleteTest(TestCase):

    def setUp(self):
        self.view = SubsetDelete.as_view()
        self.request = HttpRequest()
        self.request.method = 'GET'
        self.request.user = AnonymousUser()

        setattr(self.request, 'session', 'session')
        messages = FallbackStorage(self.request)
        setattr(self.request, '_messages', messages)

    def test_get_with_anonymous(self):
        """
        Accessing the view with AnonymousUser should redirect to the login
        page
        """

        subset = SubsetFactory.create()
        response = self.view(
            self.request,
            project_id=subset.project.id,
            subset_id=subset.id)

        self.assertEqual(response.status_code, 302)
        self.assertIn('/admin/account/login/', response['location'])
        self.assertEqual(Subset.objects.count(), 1)

    def test_get_with_user(self):
        """
        Accessing the view with normal user should render the page with an
        error message
        """

        subset = SubsetFactory.create()
        user = UserFactory.create()

        self.request.user = user
        response = self.view(
            self.request,
            project_id=subset.project.id,
            subset_id=subset.id).render()

        rendered = render_to_string(
            'base.html',
            {
                'error_description': 'Project matching query does not exist.',
                'error': 'Not found.',
                'user': user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)
        self.assertEqual(Subset.objects.count(), 1)

    def test_get_with_admin(self):
        """
        Deleting with project admin should redirect to subset list
        """

        subset = SubsetFactory.create()
        user = subset.project.creator

        self.request.user = user
        response = self.view(
            self.request,
            project_id=subset.project.id,
            subset_id=subset.id)

        self.assertEqual(response.status_code, 302)
        self.assertIn(
            reverse('admin:subset_list', args=(subset.project.id, )),
            response['location']
        )
        self.assertEqual(Subset.objects.count(), 0)

    def test_get_non_existing_with_admin(self):
        """
        Deleting with project admin when subset does not exist should render
        the page with an error message
        """

        project = ProjectFactory.create()
        user = project.creator

        self.request.user = user
        response = self.view(
            self.request,
            project_id=project.id,
            subset_id=634842156456).render()

        rendered = render_to_string(
            'base.html',
            {
                'error_description': 'Subset matching query does not exist.',
                'error': 'Not found.',
                'user': user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)

    def test_get_on_locked_project_with_admin(self):
        """
        Deleting with project admin when the project is locked should render
        the page with an error message
        """

        subset = SubsetFactory.create()
        subset.project.islocked = True
        subset.project.save()

        user = subset.project.creator

        self.request.user = user
        response = self.view(
            self.request,
            project_id=subset.project.id,
            subset_id=subset.id)

        self.assertEqual(1, Subset.objects.count())
        self.assertEqual(response.status_code, 302)
        self.assertIn(
            '/admin/projects/%s/subsets/%s/' % (subset.project.id, subset.id),
            response['location']
        )



"""Serializers for categories."""

from rest_framework.serializers import (
    ModelSerializer, ReadOnlyField, SerializerMethodField
)

from geokey.core.serializers import FieldSelectorSerializer
from geokey.categories.models import (
    Category, Field, TextField, NumericField,
    LookupField, LookupValue, MultipleLookupField, MultipleLookupValue
)


class FieldSerializer(ModelSerializer):
    """Serializer for fields."""

    fieldtype = ReadOnlyField()
    is_displayfield = SerializerMethodField()

    class Meta:
        """Meta information for serializer."""

        model = Field
        depth = 1
        fields = ('id', 'status', 'fieldtype', 'name', 'description', 'key',
                  'required', 'is_displayfield')
        read_only_fields = ('id', 'name', 'key')

    def get_is_displayfield(self, field):
        """
        Return whether the field is set as a display field for the category.

        Parameters
        ----------
        field : geokey.categories.models.field
            Field that is being serialized

        Returns
        -------
        Boolean
        """
        if field.category.display_field == field:
            return True
        else:
            return False


class TextFieldSerializer(FieldSerializer):
    """Serializer for text fields."""

    class Meta:
        """Meta information for serializer."""

        model = TextField
        depth = 1
        fields = ('id', 'status', 'fieldtype', 'name', 'description', 'key',
                  'required', 'is_displayfield', 'maxlength', 'textarea')
        read_only_fields = ('id', 'name', 'key')


class NumericFieldSerializer(FieldSerializer):
    """Serializer for numeric fields."""

    class Meta:
        """Meta information for serializer."""

        model = NumericField
        depth = 1
        fields = ('id', 'status', 'fieldtype', 'name', 'description', 'key',
                  'required', 'is_displayfield', 'minval', 'maxval')
        read_only_fields = ('id', 'name', 'key')


class LookupValueSerializer(ModelSerializer):
    """Serializer for single lookupvalue."""

    class Meta:
        """Meta information for serializer."""

        model = LookupValue
        fields = ('id', 'name', 'symbol')


class MultipleLookupValueSerializer(ModelSerializer):
    """Serializer for multiple lookup value."""

    class Meta:
        """Meta information for serializer."""

        model = MultipleLookupValue
        fields = ('id', 'name', 'symbol')


class BaseLookupSerializer(object):
    """Base serializer for both single and multiple lookup fields."""

    def get_lookupvalues(self, field):
        """
        Return serialized lookupvalues.

        Parameters
        ----------
        field : geokey.categories.models.Field
            Field of which lookupvalues are serialised

        Returns
        -------
        List
            Serialized lookupvalues
        """
        values = field.lookupvalues.filter(status='active')

        if isinstance(field, LookupField):
            serializer = LookupValueSerializer(values, many=True)
        elif isinstance(field, MultipleLookupField):
            serializer = MultipleLookupValueSerializer(values, many=True)

        return serializer.data


class LookupFieldSerializer(FieldSerializer, BaseLookupSerializer):
    """Serializer for single lookup fields."""

    lookupvalues = SerializerMethodField()

    class Meta:
        """Meta information for serializer."""

        model = LookupField
        depth = 1
        fields = ('id', 'status', 'fieldtype', 'name', 'description', 'key',
                  'required', 'is_displayfield', 'lookupvalues')
        read_only_fields = ('id', 'name', 'key')
        write_only_fields = ('status',)


class MultipleLookupFieldSerializer(FieldSerializer, BaseLookupSerializer):
    """Serializer for multiple lookup fields."""

    lookupvalues = SerializerMethodField()

    class Meta:
        """Meta information for serializer."""

        model = LookupField
        depth = 1
        fields = ('id', 'status', 'fieldtype', 'name', 'description', 'key',
                  'required', 'is_displayfield', 'lookupvalues')
        read_only_fields = ('id', 'name', 'key')
        write_only_fields = ('status',)


class CategorySerializer(FieldSelectorSerializer):
    """Serializer for categories."""

    fields = SerializerMethodField('get_fields_serialized')

    class Meta:
        """Meta information for serializer."""

        model = Category
        depth = 1
        fields = ('id', 'status', 'created_at', 'name', 'description',
                  'colour', 'symbol', 'order', 'fields')
        read_only_fields = ('id', 'name', 'created_at')

    def get_fields_serialized(self, category):
        """
        Return a list of serialized fields for the category.

        Parameters
        ----------
        category : geokey.categories.models.category
            Category of which field are serialized

        Returns
        -------
        List
            Serialized fields
        """
        fields = []

        for field in category.fields.filter(status='active'):
            if isinstance(field, TextField):
                serializer = TextFieldSerializer(field)
            elif isinstance(field, NumericField):
                serializer = NumericFieldSerializer(field)
            elif isinstance(field, LookupField):
                serializer = LookupFieldSerializer(field)
            elif isinstance(field, MultipleLookupField):
                serializer = MultipleLookupFieldSerializer(field)
            else:
                serializer = FieldSerializer(field)

            fields.append(serializer.data)

        return fields
